Package: cvar
Type: Package
Title: Compute Expected Shortfall and Value at Risk for Continuous
        Distributions
Version: 0.6
Authors@R: person(given = c("Georgi", "N."), family = "Boshnakov",
                  role = c("aut", "cre"), 
                  email = "georgi.boshnakov@manchester.ac.uk",
		     comment = c(ORCID = "0000-0003-2839-346X"))
Description: Compute expected shortfall (ES) and Value at Risk (VaR) from a
    quantile function, distribution function, random number generator,
    probability density function, or data.  ES is also known as Conditional
    Value at Risk (CVaR). Virtually any continuous distribution can be
    specified.  The functions are vectorized over the arguments. The
    computations are done directly from the definitions, see e.g. Acerbi and
    Tasche (2002) <doi:10.1111/1468-0300.00091>.  Some support for GARCH models
    is provided, as well.
URL: https://geobosh.github.io/cvar/ (doc),
        https://github.com/GeoBosh/cvar (devel)
BugReports: https://github.com/GeoBosh/cvar/issues
Imports: gbutils, Rdpack (>= 0.8)
Suggests: testthat, fGarch, PerformanceAnalytics
RdMacros: Rdpack
License: GPL (>= 2)
Collate: VaR.R cvar-package.R garch.R
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-16 15:57:19 UTC; georgi
Author: Georgi N. Boshnakov [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2839-346X>)
Maintainer: Georgi N. Boshnakov <georgi.boshnakov@manchester.ac.uk>
Repository: CRAN
Date/Publication: 2025-12-17 06:40:47 UTC
Built: R 4.6.0; ; 2026-04-27 14:19:15 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(ES,default)
S3method(ES,matrix)
S3method(ES,numeric)
S3method(VaR,default)
S3method(VaR,matrix)
S3method(VaR,numeric)
S3method(predict,garch1c1)
export(ES)
export(GarchModel)
export(VaR)
export(VaR_cdf)
export(VaR_qf)
export(sim_garch1c1)
importFrom(Rdpack,reprompt)
importFrom(gbutils,cdf2quantile)
importFrom(stats,quantile)
importFrom(stats,runif)

Configuration Variables
=======================

Configure programs through variables and actions.

## Introduction

When an application does a particular job and exits, configuration can be loaded at startup and stored in a dedicated configuration object.

When an application is long-lived this may not suffice, it may want to have:

* Its configuration changed one setting at the time.

* Complex hierarchical organization of the configuration.

* An interface where the state of the configuration can be queried and updated.

* Executable actions instead of just variables.

* Addon support which needs its own configuration mounted under a prefix.

* Support dynamic configuration created at runtime.

## Design

Configuration is the interface where a user meets and interacts with the program state. Users interact through text while programs have their preferred data types.

The goal of a configuration manager is to facilitate this interaction. This library's scope is more narrow in providing just the means to identify configurable elements and interact with them through text.

A configurable element is called a _node_. Every _node_ has some metadata such as a _name_ and a _description_. There are three types of nodes: _properties_, _lists_ and _actions_.

* A _property_ stores a variable state. It has a _default value_ and an optional _change_ callback.

* A _list_ defines a hierarchy. It contains _child nodes_. Names in a hierarchy are separated by a dot, eg: `list.foo`.

* An _action_ is node that can be invoked with an argument string. It can execute arbitrary code in its context.

The _nodes_ are ephemeral, this means that they don't store their metadata next to and interspersed with the program state. This avoids requiring long-lived borrows and has other performance benefits.

While generic implementations are provided, each node type is a trait and can have custom implementations.

This library aims to be lightweight, avoid unnecessary memory allocations and unavoidable debug strings. The ephemeral nature of the metadata allows obfuscation.

For technical documentation see the docs below.

## Future work

Implement autocomplete for identifiers and suggestions for property values and actions.

Implement helpers for enums and enum flags support.

## Usage

This crate can be found on [crates.io](https://crates.io/crates/cvar).

Documentation can be found [online](https://casualx.github.io/docs/cvar-rs/0.1.0/cvar).

In your `Cargo.toml`:

```
[dependencies]
cvar = "0.1"
```

## License

MIT, see LICENSE.txt

[![CRANStatusBadge](https://www.r-pkg.org/badges/version/cvar)](https://cran.r-project.org/package=cvar)
[![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/cvar)](https://www.r-pkg.org/pkg/cvar)
[![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/grand-total/cvar?color=blue)](https://r-pkg.org/pkg/cvar)
[![R-CMD-check](https://github.com/GeoBosh/cvar/workflows/R-CMD-check/badge.svg)](https://github.com/GeoBosh/cvar/actions)
[![codecov](https://codecov.io/gh/GeoBosh/cvar/branch/master/graph/badge.svg?token=2SW9HKG71Y)](https://app.codecov.io/gh/GeoBosh/cvar)

 
## Overview

Compute expected shortfall (ES) and Value at Risk (VaR) from a
quantile function, distribution function, random number generator or
probability density function.  ES is also known as Conditional Value
at Risk (CVaR). Virtually any continuous distribution can be
specified.  The functions are vectorised over the arguments.
The computations are done directly from the definitions, see e.g. Acerbi
and Tasche (2002). Some support for GARCH models is provided, as well.


## Installing cvar

The [latest stable version](https://cran.r-project.org/package=cvar) is on CRAN. 

    install.packages("cvar")

The vignette
[Guide_cvar]( https://CRAN.R-project.org/package=cvar/vignettes/Guide_cvar.pdf) shipping
with the package gives illustrative examples (can also be opened from R with
`vignette("Guide_cvar", package = "cvar")`).

You can install the [development version](https://github.com/GeoBosh/cvar) of `cvar` from Github:

    library(devtools)
    install_github("GeoBosh/cvar")


## Overview

Package `cvar` is a small `R` package with, essentially two
functions &#x2014; `ES` for computing the expected shortfall
and `VaR` for Value at Risk.  The user specifies the
distribution by supplying one of the functions that define a
continuous distribution&#x2014;currently this can be a quantile
function (qf), cumulative distribution function (cdf) or
probability density function (pdf). Virtually any continuous
distribution can be specified.

The functions are vectorised over the parameters of the
distributions, making bulk computations more convenient, for
example for forecasting or model evaluation.

The name of this package, "cvar", comes from *Conditional Value at
Risk* (CVaR), which is an alternative term for expected shortfall.

We chose to use the standard names `ES` and `VaR`,
despite the possibility for name clashes with same named
functions in other packages, rather than invent possibly
difficult to remember alternatives. Just call the functions as
`cvar::ES` and `cvar::VaR` if necessary.

Locations-scale transformations can be specified separately
from the other distribution parameters. This is useful when
such parameters are not provided directly by the distribution
at hand. The use of these parameters often leads to more
efficient computations and better numerical accuracy even if
the distribution has its own parameters for this purpose. Some
of the examples for `VaR` and `ES` illustrate this
for the Gaussian distribution.

Since VaR is a quantile, functions computing it for a given
distribution are convenience functions. `VaR` exported by
`cvar` could be attractive in certain workflows because of
its vectorised distribution parameters, the location-scale
transformation and the possibility to compute it from cdf's
when quantile functions are not available.

#' Compute Value-at-Risk (VaR)
#'
#' @description \code{VaR} computes the Value-at-Risk of the distribution specified by the
#'     arguments. The meaning of the parameters is the same as in \code{\link{ES}}, including
#'     the recycling rules.
#'
#' @note We use the traditional definition of VaR as the negated lower quantile. For example,
#'     if \eqn{X} are returns on an asset, VAR\eqn{{}_\alpha}{_a} = \eqn{-q_\alpha}{-q_a},
#'     where \eqn{q_\alpha}{-q_a} is the lower \eqn{\alpha}{a} quantile of \eqn{X}.
#'     Equivalently, VAR\eqn{{}_\alpha}{_a} is equal to the lower \eqn{1-\alpha}{1-a}
#'     quantile of \eqn{-X}.
#'
#' @inheritParams ES
#'
#' @param intercept,slope compute VaR for the linear transformation \code{intercept +
#'     slope*X}, where \code{X} has distribution specified by \code{dist}, see Details.
#' 
#'
#' @details
#'     \code{VaR} is S3 generic. The meaning of the parameters for its default method is the
#'     same as in \code{\link{ES}}, including the recycling rules.
#'
#'     \code{VaR_qf} and \code{VaR_cdf} are streamlined, non-generic, variants for the common
#'     case when the \code{"..."} parameters are scalar. The parameters \code{x},
#'     \code{intercept}, and \code{slope} can be vectors, as for \code{VaR}.
#'
#'     Argument \code{dist} can also be a numeric vector. In that case the ES is computed,
#'     effectively, for the empirical cumulative distribution function (ecdf) of the
#'     vector. The ecdf is not created explicitly and the \code{\link[stats]{quantile}}
#'     function is used instead for the computation of VaR. Arguments in \code{"..."} are
#'     passed eventually to \code{quantile()} and can be used, for example, to select a
#'     non-defult method for the computation of quantiles.
#'
#'     In practice, we may need to compute VaR associated with data. The distribution comes
#'     from fitting a model. In the simplest case, we fit a distribution to the data,
#'     assuming that the sample is i.i.d. For example, a normal distribution \eqn{N(\mu,
#'     \sigma^2)} can be fitted using the sample mean and sample variance as estimates of the
#'     unknown parameters \eqn{\mu} and \eqn{\sigma^2}, see section \sQuote{Examples}. For
#'     other common distributions there are specialised functions to fit their parameters and
#'     if not, general optimisation routines can be used. More soffisticated models may be
#'     used, even time series models such as GARCH and mixture autoregressive models.
#'     
#'     
#' @param tol tollerance
#'
#' @seealso \code{\link{ES}} for ES,
#'
#'     \code{\link[=predict.garch1c1]{predict}} for examples with fitted models
#' 
#' @examples
#' cvar::VaR(qnorm, c(0.01, 0.05), dist.type = "qf")
#'
#' ## the following examples use these values, obtained by fitting a normal distribution to
#' ## some data:
#' muA <- 0.006408553
#' sigma2A <- 0.0004018977
#'
#' ## with quantile function, giving the parameters directly in the call:
#' res1 <- cvar::VaR(qnorm, 0.05, mean = muA, sd = sqrt(sigma2A))
#' res2 <- cvar::VaR(qnorm, 0.05, intercept = muA, slope = sqrt(sigma2A))
#' abs((res2 - res1)) # 0, intercept/slope equivalent to mean/sd
#'
#' ## with quantile function, which already knows the parameters:
#' my_qnorm <- function(p) qnorm(p, mean = muA, sd = sqrt(sigma2A))
#' res1_alt <- cvar::VaR(my_qnorm, 0.05)
#' abs((res1_alt - res1))
#' 
#' ## with cdf the precision depends on solving an equation
#' res1a <- cvar::VaR(pnorm, 0.05, dist.type = "cdf", mean = muA, sd = sqrt(sigma2A))
#' res2a <- cvar::VaR(pnorm, 0.05, dist.type = "cdf", intercept = muA, slope = sqrt(sigma2A))
#' abs((res1a - res2)) # 3.287939e-09
#' abs((res2a - res2)) # 5.331195e-11, intercept/slope better numerically
#'
#' ## as above, but increase the precision, this is probably excessive
#' res1b <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
#'                    mean = muA, sd = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
#' res2b <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
#'                    intercept = muA, slope = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
#' abs((res1b - res2)) # 6.938894e-18 # both within machine precision
#' abs((res2b - res2)) # 1.040834e-16
#'
#' ## relative precision is also good
#' abs((res1b - res2)/res2) # 2.6119e-16 # both within machine precision
#' abs((res2b - res2)/res2) # 3.91785e-15
#'
#'
#' ## an extended example with vector args, if "PerformanceAnalytics" is present
#' if (requireNamespace("PerformanceAnalytics", quietly = TRUE)) withAutoprint({
#'     data(edhec, package = "PerformanceAnalytics")
#'     mu <- apply(edhec, 2, mean)
#'     sigma2 <- apply(edhec, 2, var)
#'     musigma2 <- cbind(mu, sigma2)
#'
#'     ## compute in 2 ways with cvar::VaR
#'     vAz1 <- cvar::VaR(qnorm, 0.05, mean = mu, sd = sqrt(sigma2))
#'     vAz2 <- cvar::VaR(qnorm, 0.05, intercept = mu, slope = sqrt(sigma2))
#'
#'     vAz1a <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
#'                        mean = mu, sd = sqrt(sigma2))
#'     vAz2a <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
#'                        intercept = mu, slope = sqrt(sigma2))
#'
#'     vAz1b <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
#'                    mean = mu, sd = sqrt(sigma2),
#'                    tol = .Machine$double.eps^0.75)
#'     vAz2b <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
#'                    intercept = mu, slope = sqrt(sigma2),
#'                    tol = .Machine$double.eps^0.75)
#'
#'     ## analogous calc. with PerformanceAnalytics::VaR
#'     vPA <- apply(musigma2, 1, function(x)
#'         PerformanceAnalytics::VaR(p = .95, method = "gaussian", invert = FALSE,
#'                                   mu = x[1], sigma = x[2], weights = 1))
#'     ## the results are numerically the same
#'     max(abs((vPA - vAz1))) # 5.551115e-17
#'     max(abs((vPA - vAz2))) #   ""
#'
#'     max(abs((vPA - vAz1a))) # 3.287941e-09
#'     max(abs((vPA - vAz2a))) #  1.465251e-10, intercept/slope better
#'
#'     max(abs((vPA - vAz1b))) # 4.374869e-13
#'     max(abs((vPA - vAz2b))) # 3.330669e-16
#' })
#'
#' @export
VaR <- function(dist, p_loss = 0.05, ...){
    UseMethod("VaR")
}

warn_about_x <- function(s){
    warning("argument 'x' has been renamed to 'p_loss'. 'x' still works with this version of 'cvar' but will stop doing so in the next release.\n Please change your scripts suitably.\n It most cases there is no need to name this argument when calling ", s)
}

#' @name VaR
#'
#' @export
VaR_qf <- function(dist, p_loss = 0.05, ...,
                   intercept = 0, slope  = 1, tol = .Machine$double.eps^0.5, x){
    if(!missing(x)){
        p_loss <- x
        warn_about_x("VaR_qf")
    }
    dist <- match.fun(dist)
    ## assumes "..." are scalar
    res <-  if(length(p_loss) > 1)
                - sapply(p_loss, dist, ...)
            else
                - dist(p_loss, ...)

    - intercept + slope * res
}

#' @name VaR
#'
#' @export
VaR_cdf <- function(dist, p_loss = 0.05, ...,
                    intercept = 0, slope  = 1, tol = .Machine$double.eps^0.5, x){
    if(!missing(x)){
        p_loss <- x
        warn_about_x("VaR_cdf")
    }
    
    dist <- match.fun(dist)
    ## assumes "..." are scalar
    res <-  if(length(p_loss) > 1)
                - sapply(p_loss, cdf2quantile, ..., MoreArgs = list(cdf = dist, tol = tol))
            else
                - cdf2quantile(p_loss, dist, tol = tol, ...) # TODO: interval?

    - intercept + slope * res
}

#' @name VaR
#'
#' @export
VaR.default <- function(dist, p_loss = 0.05, dist.type = "qf", ...,
                        intercept = 0, slope  = 1, tol = .Machine$double.eps^0.5, x){
    if(!missing(x)){
        p_loss <- x
        warn_about_x("VaR")
    }
    
    dist <- match.fun(dist)
    ## TODO: add "rand" to dist.type
    ## TODO: include 'dist' in the condition and mapply?
            # fu <- switch(dist.type,
            #              qf = function(y) - dist(y, ...),
            #              cdf = function(y) - cdf2quantile(y, dist, tol = tol, ...),
            #              pdf = {
            #                  stop("Not ready yet. Please supply quantile function or cdf")
            #              },
            #              stop('argument dist.type should be one of "qf", "cdf" or "pdf"')
            #              )
            # sapply(p_loss, fu)
    res <- switch(dist.type,
                  qf = - mapply(dist, p_loss, ...),
                  cdf = - mapply(cdf2quantile, p_loss, ..., MoreArgs = list(cdf = dist, tol = tol)),
                  pdf = {
                      stop("Not ready yet. Please supply quantile function or cdf")
                  },
                  stop('argument dist.type should be one of "qf", "cdf" or "pdf"')
                  )
    - intercept + slope * res
}

#' @name VaR
#'
#' @export
VaR.numeric <- function(dist, p_loss = 0.05, ..., intercept = 0, slope  = 1, x){
    if(!missing(x)){
        p_loss <- x
        warn_about_x("VaR")
    }
    
    ## dist is raw data here
    res <- - quantile(dist, p_loss, ...)

    - intercept + slope * res
}



## VaR <- function(dist, p_loss = 0.05, dist.type = "qf", ...,
##                 intercept = 0, slope  = 1, tol = .Machine$double.eps^0.5){
##     dist <- match.fun(dist)
##     ## TODO: add "rand" to dist.type
##     ## TODO: include 'dist' in the condition and mapply()?
##     if(length(p_loss) == 1 && all(sapply(list(...), length) < 2))
##         res <- switch(dist.type,
##                       qf = - dist(p_loss, ...),
##                       cdf = - cdf2quantile(p_loss, dist, tol = tol, ...), # TODO: interval?
##                       pdf = {
##                           stop("Not ready yet. Please supply quantile function or cdf")
##                       },
##                       stop('argument dist.type should be one of "qf", "cdf" or "pdf"')
##                       )
##     else{
##             # fu <- switch(dist.type,
##             #              qf = function(y) - dist(y, ...),
##             #              cdf = function(y) - cdf2quantile(y, dist, tol = tol, ...),
##             #              pdf = {
##             #                  stop("Not ready yet. Please supply quantile function or cdf")
##             #              },
##             #              stop('argument dist.type should be one of "qf", "cdf" or "pdf"')
##             #              )
##             # sapply(p_loss, fu)
##         res <- - switch(dist.type,
##                         qf = mapply(dist, p_loss, ...),
##                         cdf = mapply(cdf2quantile, p_loss, ..., MoreArgs = list(cdf = dist, tol = tol)),
##                         pdf = {
##                             stop("Not ready yet. Please supply quantile function or cdf")
##                         },
##                         stop('argument dist.type should be one of "qf", "cdf" or "pdf"')
##                         )
##
##     }
##     - intercept + slope * res
## }

#' @title Compute expected shortfall (ES) of distributions
#'
#' @description
#'     Compute the expected shortfall for a distribution. 
#'
#' @details \code{ES} computes the expected shortfall for distributions specified by the
#'     arguments. \code{dist} is typically a function (or the name of one). What \code{dist}
#'     computes is determined by \code{dist.type}, whose default setting is \code{"qf"} (the
#'     quantile function). Other possible settings of \code{dist.type} include \code{"cdf"}
#'     and \code{"pdf"}.  Additional arguments for \code{dist} can be given with the
#'     \code{"..."} arguments.
#'
#'     Argument \code{dist} can also be a numeric vector. In that case the ES is computed,
#'     effectively, for the empirical cumulative distribution function (ecdf) of the
#'     vector. The ecdf is not created explicitly and the \code{\link[stats]{quantile}}
#'     function is used instead for the computation of VaR. Arguments in \code{"..."} are
#'     passed eventually to \code{quantile()} and can be used, for example, to select a
#'     non-defult method for the computation of quantiles.
#'
#'     Except for the exceptions discussed below, a function computing VaR for the specified
#'     distribution is constructed and the expected shortfall is computed by numerically
#'     integrating it. The numerical integration can be fine-tuned with argument
#'     \code{control}, which should be a named list, see \code{\link{integrate}} for the
#'     available options.
#'
#'     If \code{dist.type} is \code{"pdf"}, VaR is not computed, Instead, the partial
#'     expectation of the lower tail is computed by numerical integration of \code{x *
#'     pdf(x)}.  Currently the quantile function is required anyway, via argument \code{qf},
#'     to compute the upper limit of the integral. So, this case is mainly for testing and
#'     comparison purposes.
#'
#'
#'     A bunch of expected shortfalls is computed if argument \code{x} or any of the
#'     arguments in \code{"..."} are of length greater than one. They are recycled to equal
#'     length, if necessary, using the normal \R recycling rules.
#'
#'     \code{intercept} and \code{slope} can be used to compute the expected shortfall for
#'     the location-scale transformation \code{Y = intercept + slope * X}, where the
#'     distribution of \code{X} is as specified by the other parameters and \code{Y} is the
#'     variable of interest. The expected shortfall of \code{X} is calculated and then
#'     transformed to that of \code{Y}. Note that the distribution of \code{X} doesn't need
#'     to be standardised, although it typically will.
#'
#'     The \code{intercept} and the \code{slope} can be vectors. Using them may be
#'     particularly useful for cheap calculations in, for example, forecasting, where the
#'     predictive distributions are often from the same family, but with different location
#'     and scale parameters. Conceptually, the described treatment of \code{intercept} and
#'     \code{slope} is equivalent to recycling them along with the other arguments, but more
#'     efficiently.
#'
#'     The names, \code{intercept} and \code{slope}, for the location and scale parameters
#'     were chosen for their expressiveness and to minimise the possibility for a clash with
#'     parameters of \code{dist} (e.g., the Gamma distribution has parameter \code{scale}).
#'
#'
#' @param dist specifies the distribution whose ES is computed, usually a function or a name
#'     of a function computing quantiles, cdf, pdf, or a random number generator, see
#'     Details.
#'
#' @param p_loss level, default is 0.05.
#'
#' @param dist.type a character string specifying what is computed by \code{dist}, such as
#'     "qf" or "cdf".
#'
#' @param qf quantile function, only used if \code{dist.type = "pdf"}.
#'
#' @param intercept,slope compute ES for the linear transformation \code{intercept +
#'     slope*X}, where \code{X} has distribution specified by \code{dist}, see Details.
#'
#' @param control additional control parameters for the numerical integration routine.
#'
#' @param ... passed on to \code{dist}.
#'
#' @param x deprecated and will soon be removed. \code{x} was renamed to \code{p_loss},
#'     please use the latter.
#'
#'
#' @return a numeric vector
#'
#' @seealso \code{\link{VaR}} for VaR,
#'
#'     \code{\link[=predict.garch1c1]{predict}} for examples with fitted models
#' 
#' @examples
#' ES(qnorm)
#'
#' ## Gaussian
#' ES(qnorm, dist.type = "qf")
#' ES(pnorm, dist.type = "cdf")
#'
#' ## t-dist
#' ES(qt, dist.type = "qf", df = 4)
#' ES(pt, dist.type = "cdf", df = 4)
#'
#' ES(pnorm, 0.95, dist.type = "cdf")
#' ES(qnorm, 0.95, dist.type = "qf")
#' ## - VaRES::esnormal(0.95, 0, 1)
#' ## - PerformanceAnalytics::ETL(p=0.05, method = "gaussian", mu = 0,
#' ##                             sigma = 1, weights = 1)             # same
#'
#' cvar::ES(pnorm, dist.type = "cdf")
#' cvar::ES(qnorm, dist.type = "qf")
#' cvar::ES(pnorm, 0.05, dist.type = "cdf")
#' cvar::ES(qnorm, 0.05, dist.type = "qf")
#'
#' ## this uses "pdf"
#' cvar::ES(dnorm, 0.05, dist.type = "pdf", qf = qnorm)
#'
#'
#' ## this gives warning (it does more than simply computing ES):
#' ## PerformanceAnalytics::ETL(p=0.95, method = "gaussian", mu = 0, sigma = 1, weights = 1)
#'
#' ## run this if VaRRES is present
#' \dontrun{
#' x <- seq(0.01, 0.99, length = 100)
#' y <- sapply(x, function(p) cvar::ES(qnorm, p, dist.type = "qf"))
#' yS <- sapply(x, function(p) - VaRES::esnormal(p))
#' plot(x, y)
#' lines(x, yS, col = "blue")
#' }
#'
#' @export
ES <- function(dist, p_loss = 0.05, ...){
    UseMethod("ES")
}

CVaR <- AVaR <- ES # synonyms for ES, currently not exported


#' @name ES
#'
#' @export
ES.default <- function(dist, p_loss = 0.05, dist.type = "qf", qf, ...,
                       intercept = 0, slope = 1, control = list(), x){
    if(!missing(x)){
        p_loss <- x
        warn_about_x("ES")
    }

    dist <- match.fun(dist)

    if(length(p_loss) == 1 && all(sapply(list(...), length) < 2)){
        if(dist.type == "pdf"){
            fu <- function(p) - p * dist(p, ...) / p_loss

            iargs <- list(f = fu, lower = -Inf, upper = qf(p_loss)) # stop.on.error = FALSE,?

        }else{
            fu <- function(p) VaR(dist = dist, p_loss = p, dist.type = dist.type, ...) / p_loss

            iargs <- list(f = fu, lower = 0, upper = p_loss) # stop.on.error = FALSE,?
        }

        iargs[names(control)] <- control

        res <- do.call("integrate", iargs)$value
    }else{
        ## TODO: this is lazy
        if(dist.type == "pdf"){
            if(is.list(qf))
                res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control),
                              p_loss = p_loss, qf = qf, ...)
            else
                res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control, qf = qf),
                          p_loss = p_loss, ...)
                
        }else{
            res <- mapply(ES, MoreArgs = list(dist = dist, dist.type = dist.type, control = control),
                          p_loss = p_loss, ...)
        }
    }

    -intercept + slope * res
}

#' @name ES
#'
#' @export
ES.numeric <- function(dist, p_loss = 0.05, dist.type = "qf", qf, ...,
                       intercept = 0, slope = 1, control = list(), x){
    if(!missing(x)){
        p_loss <- x
        warn_about_x("ES")
    }
    
    v <- VaR.numeric(dist, p_loss = p_loss, ..., intercept = intercept, slope = slope)
    bad <- dist[dist <= - v]
    res <- - mean(bad)

    res
}

library(cvar)
context("VaR")

test_that("VaR works ok", {

muA <- 0.006408553
sigma2A <- 0.0004018977

## with quantile function
res1 <- cvar::VaR(qnorm, p_loss = 0.05, mean = muA, sd = sqrt(sigma2A))
res2 <- cvar::VaR(qnorm, p_loss = 0.05, intercept = muA, slope = sqrt(sigma2A))
abs((res2 - res1)) # 0, intercept/slope equivalent to mean/sd

expect_error(cvar::VaR(dnorm, dist.type = "pdf", p_loss = 0.05, intercept = muA, slope = sqrt(sigma2A)), "Not ready")
cvar::VaR_qf(qnorm, p_loss = 0.05, mean = muA, sd = sqrt(sigma2A))

## with cdf the precision depends on solving an equation
res1a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf", mean = muA, sd = sqrt(sigma2A))
res2a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf", intercept = muA, slope = sqrt(sigma2A))
abs((res1a - res2)) # 3.287939e-09
abs((res2a - res2)) # 5.331195e-11, intercept/slope better numerically

expect_equal(res1, VaR_qf(qnorm, p_loss = 0.05, mean = muA, sd = sqrt(sigma2A)))
expect_equal(res2a, VaR_cdf(pnorm, p_loss = 0.05, mean = muA, sd = sqrt(sigma2A)))

 
set.seed(1236)
a.num <- rnorm(100)
VaR(a.num)

## test the fix for issue #2
expect_true(ES(a.num) == ES(a.num, p_loss = 0.05))
expect_true(ES(a.num) != ES(a.num, p_loss = 0.01))

## as above, but increase the precision, this is probably excessive
res1b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf", mean = muA, sd = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
res2b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf", intercept = muA, slope = sqrt(sigma2A), tol = .Machine$double.eps^0.75)

expect_lt( abs((res1b - res2)), 1e-12) # 6.938894e-18 # both within machine precision
expect_lt( abs((res2b - res2)), 1e-12) # 1.040834e-16

## relative precision is also good
abs((res1b - res2)/res2) # 2.6119e-16 # both within machine precision
abs((res2b - res2)/res2) # 3.91785e-15


## an extended example with vector args, if "PerformanceAnalytics" is present
if(require("PerformanceAnalytics")){
    data(edhec, package = "PerformanceAnalytics")
    mu <- apply(edhec, 2, mean)
    sigma2 <- apply(edhec, 2, var)
    musigma2 <- cbind(mu, sigma2)

    ## compute in 2 ways with cvar::VaR
    vAz1 <- cvar::VaR(qnorm, p_loss = 0.05, mean = mu, sd = sqrt(sigma2))
    vAz2 <- cvar::VaR(qnorm, p_loss = 0.05, intercept = mu, slope = sqrt(sigma2))

    vAz1a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                       mean = mu, sd = sqrt(sigma2))
    vAz2a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                       intercept = mu, slope = sqrt(sigma2))

    vAz1b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = mu, sd = sqrt(sigma2),
                   tol = .Machine$double.eps^0.75)
    vAz2b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = mu, slope = sqrt(sigma2),
                   tol = .Machine$double.eps^0.75)

    ## analogous calc. with PerformanceAnalytics::VaR
    vPA <- apply(musigma2, 1, function(x)
        PerformanceAnalytics::VaR(p = .95, method = "gaussian", invert = FALSE,
                                  mu = x[1], sigma = x[2], weights = 1))
    ## the results are numerically the same
    max(abs((vPA - vAz1))) # 5.551115e-17
    max(abs((vPA - vAz2))) #   ""

    max(abs((vPA - vAz1a))) # 3.287941e-09
    max(abs((vPA - vAz2a))) #  1.465251e-10, intercept/slope better

    max(abs((vPA - vAz1b))) # 4.374869e-13
    max(abs((vPA - vAz2b))) # 3.330669e-16
}

})

test_that("ES works ok", {

expect_equal(ES(qnorm, dist.type = "qf"),
             ES(qnorm) )

## Gaussian
expect_equal(ES(qnorm, dist.type = "qf"),
             ES(pnorm, dist.type = "cdf") )


## t-dist
expect_equal(ES(qt, dist.type = "qf", df = 4),
             ES(pt, dist.type = "cdf", df = 4) )

expect_equal(ES(pnorm, p_loss = 0.95, dist.type = "cdf"),
             ES(qnorm, p_loss = 0.95, dist.type = "qf") )
## - VaRES::esnormal(0.95, 0, 1)
## - PerformanceAnalytics::ETL(p=0.05, method = "gaussian", mu = 0,
##                             sigma = 1, weights = 1)             # same


## this uses "pdf"
expect_equal(ES(dnorm, p_loss = 0.05, dist.type = "pdf", qf = qnorm),
             ES(pnorm, p_loss = 0.05, dist.type = "cdf") )

## several in one call
ES(pnorm, p_loss = c(0.1, 0.05, 0.01), dist.type = "cdf")
ES(dnorm, p_loss = c(0.1, 0.05, 0.01), dist.type = "pdf", qf = qnorm)
ES(dnorm, p_loss = 0.05, dist.type = "pdf", qf = qnorm)
ES(dnorm, p_loss = c(0.1, 0.05, 0.01), dist.type = "pdf", qf = qnorm)
ES(dnorm, p_loss = c(0.1, 0.05, 0.01), dist.type = "pdf", qf = list(qnorm, qnorm, qnorm))
ES(dnorm, p_loss = c(0.1, 0.05, 0.01), dist.type = "pdf", qf = c(qnorm, qnorm, qnorm))
})

#' Specify a GARCH model
#'
#' Specify a GARCH model.
#'
#' Argument \code{model} can be the result of a previous call to \code{GarchModel}.
#' Arguments in \code{"..."} overwrite current components of \code{model}.
#'
#' \code{GarchModel} guarantees that code using it will continue to work
#' transparently for the user even if the internal represedtation of GARCH
#' models in this package is changed or additional functionality is added.
#'
#' @param model a GARCH model or a list.
#' @param ... named arguments specifying the GARCH model.
#' @param model.class a class for the result. By default \code{GarchModel()}
#'     decides the class of the result.
#' @return an object from suitable GARCH-type class
#'
#' @examples
#' ## GARCH(1,1) with Gaussian innovations
#' mo1a <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5)
#' mo1b <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5, cond.dist = "norm")
#'
#' ## equivalently, the parameters can be given as a list
#' p1 <- list(omega = 1, alpha = 0.3, beta = 0.5)
#' mo1a_alt <- GarchModel(p1)
#' mo1b_alt <- GarchModel(p1, cond.dist = "norm")
#' stopifnot(identical(mo1a, mo1a_alt), identical(mo1b, mo1b_alt))
#'
#' ## additional arguments modify values already in 'model'
#' mo_alt <- GarchModel(p1, beta = 0.4)
#'
#' ## set also initial values
#' mo2 <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5, esp0 = - 1.5, h0 = 4.96)
#'
#' ##  GARCH(1,1) with standardised-t_5
#' mot <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5, cond.dist = list("std", nu = 5))
#'
#' @export
GarchModel <- function(model = list(), ..., model.class = NULL){
    ## TODO: check the correctness of the parameters
    ## alpha, beta, cond.dist, 2nd order stationary, initial value for eps_t^2, h_t

    ## 2019-03-15 TODO: handle fGARCH models
    ## if(inherits(model, "fGARCH")){
    ##
    ## }else{

        dots <- list(...)
        if(length(dots) > 0)
            model[names(dots)] <- dots

        class(model) <- "GarchModel0"
        if(is.null(model.class) && length(model$alpha) == 1 && length(model$beta) == 1)
            class(model) <- "garch1c1"                     # GARCH(1,1), 'c' stands for 'comma'
    ## }

    model
}

## 2022-11-01 change .dist from 'list' to 'env', was:
    # .dist <- list(
    #     norm = list(d = call("dnorm", x = NA, mean = 0, sd = 1),
    #                 p = call("pnorm", q = NA, mean = 0, sd = 1),
    #                 q = call("qnorm", p = NA, mean = 0, sd = 1),
    #                 r = call("rnorm", n = NA, mean = 0, sd = 1)),
    #     std  = list(d = call("dstd",  x = NA, mean = 0, sd = 1, nu = NA),
    #                 p = call("pstd",  q = NA, mean = 0, sd = 1, nu = NA),
    #                 q = call("qstd",  p = NA, mean = 0, sd = 1, nu = NA),
    #                 r = call("rstd",  n = NA, mean = 0, sd = 1, nu = NA)),
    #     ged  = list(d = call("dged",  x = NA, mean = 0, sd = 1, nu = NA),
    #                 p = call("pged",  q = NA, mean = 0, sd = 1, nu = NA),
    #                 q = call("qged",  p = NA, mean = 0, sd = 1, nu = NA),
    #                 r = call("rged",  n = NA, mean = 0, sd = 1, nu = NA))
    # )

.dist <- new.env()

## N(0,1) is built-in
.dist$norm <- list(d = call("dnorm", x = NA, mean = 0, sd = 1),
                   p = call("pnorm", q = NA, mean = 0, sd = 1),
                   q = call("qnorm", p = NA, mean = 0, sd = 1),
                   r = call("rnorm", n = NA, mean = 0, sd = 1))


    # generalise and prepare to make fGarch suggested; was:
    # 
    # .dist$std  <- list(d = call("dstd",  x = NA, mean = 0, sd = 1, nu = NA),
    #                    p = call("pstd",  q = NA, mean = 0, sd = 1, nu = NA),
    #                    q = call("qstd",  p = NA, mean = 0, sd = 1, nu = NA),
    #                    r = call("rstd",  n = NA, mean = 0, sd = 1, nu = NA))
    # 
    # .dist$ged  <- list(d = call("dged",  x = NA, mean = 0, sd = 1, nu = NA),
    #                    p = call("pged",  q = NA, mean = 0, sd = 1, nu = NA),
    #                    q = call("qged",  p = NA, mean = 0, sd = 1, nu = NA),
    #                    r = call("rged",  n = NA, mean = 0, sd = 1, nu = NA))

.dist_list <- list(
    ## TODO: allow different packages here by letting 'package be vector or completely
    ##       refactor all this. Only '.get_dist_elem()' should know the implementation.
    std = list(package = "fGarch",
               d = list(str2lang("fGarch::dstd"), x = NA, mean = 0, sd = 1, nu = NA),
               p = list(str2lang("fGarch::pstd"), q = NA, mean = 0, sd = 1, nu = NA),
               q = list(str2lang("fGarch::qstd"), p = NA, mean = 0, sd = 1, nu = NA),
               r = list(str2lang("fGarch::rstd"), n = NA, mean = 0, sd = 1, nu = NA)),

    ged = list(package = "fGarch",
               d = list(str2lang("fGarch::dged"), x = NA, mean = 0, sd = 1, nu = NA),
               p = list(str2lang("fGarch::pged"), q = NA, mean = 0, sd = 1, nu = NA),
               q = list(str2lang("fGarch::qged"), p = NA, mean = 0, sd = 1, nu = NA),
               r = list(str2lang("fGarch::rged"), n = NA, mean = 0, sd = 1, nu = NA))
)

.get_dist_elem <- function(dist, what){
    if(is.list(dist)){
        params <- dist[-1]
        dist <- dist[[1]]
    }else
        params <- list()

    if(is.null(dist)){  ## TODO: this probably should be handled by the caller
        ## normal distribution is default
        res <- .dist$norm[[what]]
    }else{
        stopifnot(is.character(dist), length(dist) == 1)

        gen <- .dist[[dist]]
        if(!is.null(gen))  # already prepared
            res <- gen[[what]]
        else{ # not prepared yet
            gen_list <- .dist_list[[dist]]
            if(is.null(gen_list))
                stop("unknown distribution specified.")

            if(!requireNamespace(gen_list$package))
                stop("requested distribution property needs package ", gen_list$package,
                     "\nplease install package '", gen_list$package, "' and try again")
    
            ## prepare
            ## .dist is env, so assignment is persistent
            .dist[[dist]] <- lapply(gen_list[-1], as.call) # dropping $package
            res <- .dist[[dist]][[what]]
            if(is.null(res))
                stop("property '", "'what not available for '", dist, "'")
        }
    }

    if(length(params) > 0)  # set parameters if specified
        res[names(params)] <- params

    res
}

#' Simulate GARCH(1,1) time series
#'
#' Simulate GARCH(1,1) time series.
#'
#' The simulated time series is in component \code{eps} of the returned value.
#' For exploration of algorithms and eestimation procedures, the volatilities
#' and the standardised innovations are also returned.
#'
#' The random seed at the start of the simulations is saved in the returned
#' object.  A speficific seed can be requested with argument \code{seed}. In
#' that case the simulations are done with the specified seed and the old state
#' of the random number generator is restored before the function returns.
#'
#' @param model a GARCH(1,1) model, an object obtained from \code{GarchModel}.
#' @param n the length of the generated time series.
#' @param n.start number of warm-up values, which are then dropped.
#' @param seed an integer to use for setting the random number generator.
#'
#' @return a list with components:
#' \item{eps}{the time series,}
#' \item{h}{the (squared) volatilities,}
#' \item{eta}{the standardised innovations,}
#' \item{model}{the GARCH(1,1) model,}
#' \item{.sim}{a list containing the parameters of the simulation,}
#' \item{call}{the call.}
#'
#' @note This function is under development and may be changed.
#'
#' @export
sim_garch1c1 <- function(model, n, n.start = 0, seed = NULL){
    ## TODO: seed is not used currently

    garch1c1_fields <- c("omega", "alpha", "beta", "cond.dist", "eps0", "h0", "eps0sq")

    stopifnot(all(names(model) %in% garch1c1_fields))

    omega  <- model$omega
    alpha  <- model$alpha
    beta   <- model$beta
    rgen   <- .get_dist_elem(model$cond.dist, "r")

    eps0   <- model$eps0
    h0     <- model$h0
    eps0sq <- model$eps0sq

    ## see Francq & Zakoian, p. 142, for alternative initialisations
    if(is.null(eps0sq)){
        model$eps0sq <- eps0sq <-
            if(is.null(eps0))
                omega / (1 - alpha - beta)
            else
                eps0^2
    }

    if(is.null(h0)){
        model$h0 <- h0 <-
            omega / (1 - alpha - beta)
    }

    N <- n + n.start
    h <- eps <- numeric(N)

    RNGstate <- .RNGstate(seed)
    ## 2020-03-04 was:    if(!is.null(RNGstate$oldRNGstate)) # or !is.null(seed)
    ##    (see comment at a similar command)
    if(!is.null(seed))
        on.exit(
            if(is.null(RNGstate$RNGstate)){
                ## TRUE id seed is NULL but also if .Random.seed was not set.
                "nothing to do"
            }else if(is.null(RNGstate$oldRNGstate)){
                rm(".Random.seed", envir = .GlobalEnv)
            }else{
                assign(".Random.seed", RNGstate$oldRNGstate, envir = .GlobalEnv)
            }
        )

    rgen$n <- N # or: rgen[[2]] <- N
    eta <- eval(rgen)

    h[1] <- omega + alpha * eps0sq + beta * h0
    for(i in 1:(N - 1)){
        eps[i] <- sqrt(h[i]) * eta[i]
        h[i + 1] <- omega + alpha * eps[i]^2 + beta * h[i]
    }
    eps[N] <- sqrt(h[N]) * eta[N]

    ## TODO: return eta only conditionally?
    ## TODO: include the model in the returned value?
    ## TODO: return also the initialisation values (in a separate component)?
    list(eps = eps[(n.start + 1):N], h = h[(n.start + 1):N], eta = eta[(n.start + 1):N],
         model = model,
         .sim = list(seed = seed, rand.gen = rgen, n = n, n.start = n.start),
         call = match.call()
         )
}

## modelled after the beginning of `stats:::simulate.lm()`
.RNGstate <- function(seed = NULL){
    ## 2020-03-04 - changing the logic below
    ##
    ## if(!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
    ##     runif(1)

    oldRNGstate <- if(exists(".Random.seed", envir = .GlobalEnv))
                       get(".Random.seed", envir = .GlobalEnv)
                   else
                       NULL
    
    if (is.null(seed))
        list(RNGstate = oldRNGstate)
    else{
        ## This doesn't resolve the problem
        ## suppressWarnings(RNGversion("3.5.0"))  # 2019-03-13 temporary, RNG changed in R-devel.
        ##                                        #            see email from Kurt Hornik in Org/
        set.seed(seed)
        RNGstate <- structure(seed, kind = as.list(RNGkind()))
        ## on.exit(assign(".Random.seed", oldRNGstate, envir = .GlobalEnv))
        list(RNGstate = RNGstate, oldRNGstate = oldRNGstate)
    }
}

#' Prediction for GARCH(1,1) time series
#'
#' Predict GARCH(1,1) time series.
#'
#' Plug-in prediction intervals and predictive distributions are obtained by
#' inserting the predicted volatility in the conditional densities. For
#' predictions more than one lag ahead these are not the real predictive
#' distributions but the prediction intervals are usually adequate.
#'
#' For simulation prediction intervals we generate a (large) number of
#' continuations of the given time series. Prediction intervals can be based on
#' sample quantiles. The generated samples are stored in the returned object and
#' can be used for further exploration of the predictive
#' distributions. \code{dist_sim$eps} contains the simulated future values of
#' the time series and \code{dist_sim$h} the corresponding (squared)
#' volatilities.  Both are matrices whose \code{i}-th rows contain the predicted
#' quantities for horizon \code{i}.
#'
#' The random seed at the start of the simulations is saved in the returned
#' object.  A speficific seed can be requested with argument \code{seed}. In
#' that case the simulations are done with the specified seed and the old state
#' of the random number generator is restored before the function returns.
#' This setup is similar to \code{\link{sim_garch1c1}}.
#'
#' @param object an object from class \code{"garch1c1"}.
#' @param n.ahead maximum horizon (lead time) for prediction.
#' @param Nsim number of Monte Carlo simulations for simulation based quantities.
#' @param eps the time series to predict, only the last value is used.
#' @param sigmasq the (squared) volatilities, only the last value is used.
#' @param seed an integer, seed for the random number generator.
#' @param ... currently not used.
#'
#' @return an object from S3 class \code{"predict_garch1c1"} containing
#'     the following components:
#' \item{eps}{point predictions (conditional expectations) of the time series (equal
#'     to zero for pure GARCH).}
#' \item{h}{point predictions (conditional expectations)of the squared volatilities.}
#' \item{model}{the model.}
#' \item{call}{the call.}
#' \item{pi_plugin}{Prediction intervals for the time series, based on plug-in
#'     distributions, see Details.}
#' \item{pi_sim}{Simulation based prediction intervals for the time series, see Details.}
#' \item{dist_sim}{simulation samples from the predictive distributions of the time
#'     series and the volatilties. }
#'
#' @examples
#' op <- options(digits = 4)
#' 
#' ## set up a model and simulate a time series
#' mo <- GarchModel(omega = 0.4, alpha = 0.3, beta = 0.5)
#' a1 <- sim_garch1c1(mo, n = 1000, n.start = 100, seed = 20220305)
#'
#' ## predictions for T+1,...,T+5 (T = time of last value)
#' ## Nsim is small to reduce the load on CRAN, usually Nsim is larger.
#' a.pred <- predict(mo, n.ahead = 5, Nsim = 1000, eps = a1$eps,
#'   sigmasq = a1$h, seed = 1234)
#'
#' ## preditions for the time series
#' a.pred$eps
#'
#' ## PI's for eps - plug-in and simulated
#' a.pred$pi_plugin
#' a.pred$pi_sim
#'
#' ## a DIY calculation of PI's using the simulated sample paths
#' t(apply(a.pred$dist_sim$eps, 1, function(x) quantile(x, c(0.025, 0.975))))
#'
#' ## further investigate the predictive distributions
#' t(apply(a.pred$dist_sim$eps, 1, function(x) summary(x)))
#'
#' ## compare predictive densities for horizons 2 and 5:
#' h2 <- a.pred$dist_sim$eps[2, ]
#' h5 <- a.pred$dist_sim$eps[5, ]
#' main <- "Predictive densities: horizons 2 (blue) and 5 (black)"
#' plot(density(h5), main = main)
#' lines(density(h2), col = "blue")
#'
#' ## predictions of sigma_t^2
#' a.pred$h
#'
#' ## plug-in predictions of sigma_t
#' sqrt(a.pred$h)
#'
#' ## simulation predictive densities (PD's) of sigma_t for horizons 2 and 5:
#' h2 <- sqrt(a.pred$dist_sim$h[2, ])
#' h5 <- sqrt(a.pred$dist_sim$h[5, ])
#' main <- "PD's of sigma_t for horizons 2 (blue) and 5 (black)"
#' plot(density(h2), col = "blue", main = main)
#' lines(density(h5))
#'
#' ## VaR and ES for different horizons
#' cbind(h = 1:5,
#'       VaR = apply(a.pred$dist_sim$eps, 1, function(x) VaR(x, c(0.05))),
#'       ES = apply(a.pred$dist_sim$eps, 1, function(x) ES(x, c(0.05))) )
#'
#' ## fit a GARCH(1,1) model to exchange rate data and predict
#' gmo1 <- fGarch::garchFit(formula = ~garch(1, 1), data = fGarch::dem2gbp,
#'   include.mean = FALSE, cond.dist = "norm", trace = FALSE)
#' mocoef <- gmo1@fit$par
#' mofitted <- GarchModel(omega = mocoef["omega"], alpha = mocoef["alpha1"],
#'   beta = mocoef["beta1"])
#' gmo1.pred <- predict(mofitted, n.ahead = 5, Nsim = 1000, eps = gmo1@data,
#'   sigmasq = gmo1@h.t, seed = 1234)
#' gmo1.pred$pi_plugin
#' gmo1.pred$pi_sim
#'
#' op <- options(op) # restore options(digits)
#'
#' @note This function is under development and may be changed.
#'
#' @export
predict.garch1c1 <- function(object, n.ahead = 1, Nsim = 1000, eps, sigmasq, seed = NULL, ...){
    model <- object
    omega <- model$omega
    alpha <- model$alpha
    beta  <- model$beta

    model$eps0 <- eps0 <- eps[length(eps)]
    model$h0   <- h0   <- sigmasq[length(sigmasq)] # TODO: compute if not present?

    ## TODO: add non-zero mean and possibly ARMA conditional mean
    pred_eps <- numeric(n.ahead)

    ## tsff slides, p. II.7.4/y1718
    pred_h <- numeric(n.ahead)
    pred_h[1] <- omega + alpha * eps0^2 + beta * h0
    if(n.ahead > 1)
        for(i in 2:n.ahead){
            pred_h[i] <- omega + (alpha + beta) * pred_h[i - 1]
        }

    fq <- .get_dist_elem(model$cond.dist, "q")
    fq$sd <- sqrt(pred_h)

    fq[[2]] <- 0.025
    lwr <- eval(fq)
    fq[[2]] <- 0.975
    upr <- eval(fq)

    eps_pi <- cbind(lwr, upr)

    ## need local variants of withr::with_seed() withr::with_preserve_seed()
    ## but such are not available (as of 2018-09-28), so do it DIY:
    ##
    ## Memorise the state in any case but restore the old state only if 'seed' is not NULL.
    RNGstate <- .RNGstate(seed)
    ## 2020-03-04 was:
    ## 
    ##    if(!is.null(RNGstate$oldRNGstate)) # or !is.null(seed)
    ##
    ## But the check is not equivalent to is.null(seed)!
    ## Moreover, if RNGstate$oldRNGstate is NULL, .Random.seed should be removed,
    ##     (so doing it now)
    if(!is.null(seed))
        on.exit(
            if(is.null(RNGstate$RNGstate)){
                ## TRUE id seed is NULL but also if .Random.seed was not set.
                "nothing to do"
            }else if(is.null(RNGstate$oldRNGstate)){
                rm(".Random.seed", envir = .GlobalEnv)
            }else{
                assign(".Random.seed", RNGstate$oldRNGstate, envir = .GlobalEnv)
            }
        )


    h_sim <- eps_sim <- matrix(NA_real_, nrow = n.ahead, ncol = Nsim)
    for(i in 1:Nsim){
        wrk <- sim_garch1c1(model = model, n = n.ahead, n.start = 0)
        eps_sim[ , i] <- wrk$eps
        h_sim[ , i] <- wrk$h
    }


    sim_pi <- t(apply(eps_sim, 1, function(x) quantile(x, c(0.025, 0.975))))

    structure(list(
        eps = pred_eps, h = pred_h, model = model, call = match.call(),
        pi_plugin = eps_pi, pi_sim = sim_pi,
        dist_sim = list(eps = eps_sim, h = h_sim, RNGstate = RNGstate$RNGstate)
    ), class = "predict_garch1c1")
}

library(cvar)
context("garch")

test_that("GarchModel works ok", {
    mo1a <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5)
    expect_equal(class(mo1a), "garch1c1")

    mo1b <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5, cond.dist = "norm")

    identical(GarchModel(mo1a, omega = 0.4),
              GarchModel(      omega = 0.4, alpha = 0.3, beta = 0.5) )

    ##  GARCH(1,1) with standardised-t_5
    mo_t <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5, cond.dist = list("std", nu = 5))
    expect_identical(mo_t, GarchModel(mo1a, cond.dist = list("std", nu = 5)))
})

test_that("garch1c1 related functions work ok", {
    ## try to deal with the misterious error on TravisCI and devtools::test()
    ## set.seed(123)
    
    a_mo <- GarchModel(omega = 0.4, alpha = 0.3, beta = 0.5)
    a <- sim_garch1c1(a_mo, n = 100, n.start = 100, seed = 1234)
    a_pred <- predict(a_mo, n.ahead = 5, Nsim = 100, eps = a$eps, sigmasq = a$h, seed = 1235)

    ## 2019-03-13 deal with change in RNG in R-devel (for 3.6.0), see email from Kurt Hornik in Org/
    ##        I couldn't make his suggestion work.
    ## TODO: maybe it would be better to just load 'a' from a saved version.
    if(getRversion() < "3.6.0"){
        a_saved <- "a_before_6.0.RDS"
        a_pred_saved <- "a_pred_before_6.0.RDS"
    }else{
        a_saved <- "a.RDS"
        a_pred_saved <- "a_pred.RDS"
    }

    expect_equal_to_reference(a, a_saved)
    expect_equal_to_reference(a_pred, a_pred_saved)

    ## as above but without 'seed'
    sim_garch1c1(a_mo, n = 100, n.start = 100)
    predict(a_mo, n.ahead = 5, Nsim = 100, eps = a$eps, sigmasq = a$h)

    expect_equal(.get_dist_elem(NULL, "r"), .dist$norm$r)
    ## for now
    expect_error(.get_dist_elem("unknowndist", "r"))
    expect_error(.get_dist_elem(list("unknowndist"), "r"))
    expect_error(.get_dist_elem(5, "r"))

    .get_dist_elem(list("norm"), "r")
    .get_dist_elem(list("norm", mean = 5, sd = 3), "r")
    .get_dist_elem(list("norm", mean = 5, sd = 3, n = 10), "r")

    .get_dist_elem("std", "r")
    .get_dist_elem(list("std", df = 5), "r")
    .get_dist_elem(list("std", df = 5, n = 10), "r")

    .get_dist_elem(list("ged"), "r")

    expect_equal(.get_dist_elem(NULL, "p"), .dist$norm[["p"]])

    .get_dist_elem("norm", "d")
    .get_dist_elem("norm", "p")
    .get_dist_elem("norm", "q")
    .get_dist_elem("norm", "r")

    .get_dist_elem(list("norm"), "p")
    .get_dist_elem(list("norm", mean = 4, sd = 2), "p")
    
    ## for now
    expect_error(.get_dist_elem("unknowndist"))
    expect_error(.get_dist_elem(list("unknowndist")))
    expect_error(.get_dist_elem(5))

})

#' @author Georgi N. Boshnakov
#'
#' @importFrom stats quantile runif
#' @importFrom gbutils cdf2quantile
#' @importFrom Rdpack reprompt
#'
#'
#'
#' @docType package
#' @name cvar-package
#' @aliases cvar
#'
#' @title Compute Conditional Value-at-Risk and Value-at-Risk
#'
#' @description Compute expected shortfall (ES) and Value at Risk (VaR)
#'     from a quantile function, distribution function, random number
#'     generator or probability density function.  ES is also known as
#'     Conditional Value at Risk (CVaR). Virtually any continuous
#'     distribution can be specified.  The functions are vectorised over
#'     the arguments. Some support for GARCH models is provided, as well.
#'
#' @details
#'
#'     There is a huge number of functions for computations with
#'     distributions in core \R and in contributed packages. Pdf's,
#'     cdf's, quantile functions and random number generators are
#'     covered comprehensively. The coverage of expected shortfall is
#'     more patchy but a large collection of distributions, including
#'     functions for expected shortfall, is provided by
#'     \insertCite{VaRES2013;textual}{cvar}.
#'     \insertCite{PerformanceAnalytics2018;textual}{cvar} and
#'     \insertCite{actuarJSS2008;textual}{cvar} provide packages
#'     covering comprehensively various aspects of risk measurement,
#'     including some functions for expected shortfall.
#'
#'     Package \pkg{cvar} is a small package with, essentially, two main
#'     functions --- \code{ES} for computing the expected shortfall
#'     and \code{VaR} for Value at Risk.  The user specifies the
#'     distribution by supplying one of the functions that define a
#'     continuous distribution---currently this can be a quantile
#'     function (qf), cumulative distribution function (cdf) or
#'     probability density function (pdf). Virtually any continuous
#'     distribution can be specified.
#'
#'     The functions are vectorised over the parameters of the
#'     distributions, making bulk computations more convenient, for
#'     example for forecasting or model evaluation.
#'
#'     The name of this package, "cvar", comes from \emph{Conditional
#'     Value at Risk} (CVaR), which is an alternative term for
#'     expected shortfall.
#'
#'     We chose to use the standard names \code{ES} and \code{VaR},
#'     despite the possibility for name clashes with same named
#'     functions in other packages, rather than invent possibly
#'     difficult to remember alternatives. Just call the functions as
#'     \code{cvar::ES} and \code{cvar::VaR} if necessary.
#'
#'     Locations-scale transformations can be specified separately
#'     from the other distribution parameters. This is useful when
#'     such parameters are not provided directly by the distribution
#'     at hand. The use of these parameters often leads to more
#'     efficient computations and better numerical accuracy even if
#'     the distribution has its own parameters for this purpose. Some
#'     of the examples for \code{VaR} and \code{ES} illustrate this
#'     for the Gaussian distribution.
#'
#'     Since VaR is a quantile, functions computing it for a given
#'     distribution are convenience functions. \code{VaR} exported by
#'     \pkg{cvar} could be attractive in certain workflows because of
#'     its vectorised distribution parameters, the location-scale
#'     transformation, and the possibility to compute it from cdf's
#'     when quantile functions are not available.
#'
#'     Some support for GARCH models is provided, as well. It is
#'     currently under development, see \code{\link{predict.garch1c1}}
#'     for current functionality.
#'
#'     In practice, we may need to compute VaR associated with data. The distribution comes
#'     from fitting a model. In the simplest case, we fit a distribution to the data,
#'     assuming that the sample is i.i.d. For example, a normal distribution \eqn{N(\mu,
#'     \sigma^2)} can be fitted using the sample mean and sample variance as estimates of the
#'     unknown parameters \eqn{\mu} and \eqn{\sigma^2}, see section \sQuote{Examples}. For other
#'     common distributions there are specialised functions to fit their parameters and if
#'     not, general optimisation routines can be used. More soffisticated models may be used,
#'     even time series models such as GARCH and mixture autoregressive models.
#'
#' @references
#'   \insertAllCited{}
#'
#' @seealso
#'   \code{\link{ES}},
#'   \code{\link{VaR}}
#' 
#' @examples
#' ## see the examples for ES(), VaR(), predict.garch1c1()
#'
NULL

.onLoad <- function(lib, pkg){
    Rdpack::Rdpack_bibstyles(package = pkg, authors = "LongNames")
    invisible(NULL)
}

% preamble taken from the vignette in package strucchange
%
\documentclass[nojss,article]{jss}
\usepackage[T1]{fontenc}
%\usepackage{a4wide}
\usepackage[left=2cm,right=2cm,bottom=15mm]{geometry}
\usepackage{graphicx,color,alltt}
\usepackage[authoryear,round,longnamesfirst]{natbib}
\usepackage{hyperref}

\usepackage{amsmath, amsfonts}
% \usepackage{Sweave}
%\definecolor{Red}{rgb}{0.7,0,0}
%\definecolor{Blue}{rgb}{0,0,0.8}
%\definecolor{hellgrau}{rgb}{0.55,0.55,0.55}

%\newcommand{\E}{\mbox{$\mathsf{E}$}}
%\newcommand{\VAR}{\mbox{$\mathsf{VAR}$}}
%\newcommand{\COV}{\mbox{$\mathsf{COV}$}}
%\newcommand{\p}{\mbox{$\mathsf{P}$}}
%\newcommand{\email}[1]{\href{mailto:#1}{\normalfont\texttt{#1}}}
%\newenvironment{smallexample}{\begin{alltt}\small}{\end{alltt}}

%\setlength{\parskip}{0.5ex plus0.1ex minus0.1ex}
%\setlength{\parindent}{0em}
%
%\bibpunct{(}{)}{;}{a}{}{,}
%
%\newcommand{\ui}{\underline{i}}
%\newcommand{\oi}{\overline{\imath}}

%\RequirePackage{color}
%\definecolor{Red}{rgb}{0.5,0,0}
%\definecolor{Blue}{rgb}{0,0,0.5}
%\definecolor{hellgrau}{rgb}{0.55,0.55,0.55}
%
%\hypersetup{%
%hyperindex,%
%colorlinks,%
%linktocpage,%
%plainpages=false,%
%linkcolor=Blue,%
%citecolor=Blue,%
%urlcolor=Red,%
%pdfstartview=Fit,%
%pdfview={XYZ null null null}%
%}


% SweaveOpts{engine=R,eps=FALSE}
%\VignetteIndexEntry{Brief guide to R package cvar}
%\VignetteDepends{cvar}
%\VignetteKeywords{expected shortfall, ES, CVaR, VaR, value at risk}
%\VignettePackage{cvar}


<<echo=FALSE>>=
library(cvar)
pd <- packageDescription("cvar")
@

\newcommand{\VaR}[2][\alpha]{\text{VaR}_{#1}(#2)}
\newcommand{\ES}[2][\alpha]{\text{ES}_{#1}(#2)}


\title{Brief guide to R package \pkg{cvar}}
\author{Georgi N. Boshnakov \\ University of Manchester }
\Plainauthor{Georgi N. Boshnakov}

\Address{
  Georgi N. Boshnakov\\
  School of Mathematics\\
  The University of Manchester\\
  Oxford Road, Manchester M13 9PL, UK\\
  URL: \url{https://personalpages.manchester.ac.uk/staff/georgi.boshnakov/}
}

%\date{}
%\maketitle


\Abstract{
Package \pkg{cvar} is a small package with, essentially, two
functions --- \code{ES()} for computing the expected shortfall
and \code{VaR()} for Value at Risk.  The user specifies the
distribution by supplying one of the functions that define a
continuous distribution---currently this can be a quantile
function (qf), cumulative distribution function (cdf) or
probability density function (pdf). Virtually any continuous
distribution can be specified.

This vignette is part of package \pkg{cvar}, version~\Sexpr{pd$Version}. %$
}

\Keywords{expected shortfall, ES, CVaR, VaR, value at risk}
\Plainkeywords{expected shortfall, ES, CVaR, VaR, value at risk}


\begin{document}

\section{Introduction}
\label{sec:introduction}

There is a huge number of functions for computations with
distributions in core R and in contributed packages. Pdf's,
cdf's, quantile functions and random number generators are
covered comprehensively. The coverage of expected shortfall is
more patchy but a large collection of distributions, including
functions for expected shortfall, is provided by
\citet{VaRES2013}.
\citet{PerformanceAnalytics2018} and
\citet{actuarJSS2008} provide packages
covering comprehensively various aspects of risk measurement,
including some functions for expected shortfall.

Package \pkg{cvar} is a small package with, essentially two
functions --- \code{ES} for computing the expected shortfall
and \code{VaR} for Value at Risk.  The user specifies the
distribution by supplying one of the functions that define a
continuous distribution---currently this can be a quantile
function (qf), cumulative distribution function (cdf) or
probability density function (pdf). Virtually any continuous
distribution can be specified.
The computations are done directly from the definitions, see e.g. 
\citet{acerbi2002expected}.

The functions are vectorised over the parameters of the
distributions, making bulk computations more convenient, for
example for forecasting or model evaluation.

The name of this package, "cvar", comes from \emph{Conditional
Value at Risk} (CVaR), which is an alternative term for
expected shortfall.

We chose to use the standard names \code{ES} and \code{VaR},
despite the possibility for name clashes with same named
functions in other packages, rather than invent possibly
difficult to remember alternatives. Just call the functions as
\code{cvar::ES} and \code{cvar::VaR} if necessary.

Locations-scale transformations can be specified separately
from the other distribution parameters. This is useful when
such parameters are not provided directly by the distribution
at hand. The use of these parameters often leads to more
efficient computations and better numerical accuracy even if
the distribution has its own parameters for this purpose. Some
of the examples for \code{VaR} and \code{ES} illustrate this
for the Gaussian distribution.

Since VaR is a quantile, functions computing it for a given
distribution are convenience functions. \code{VaR} exported by
\pkg{cvar} could be attractive in certain workflows because of
its vectorised distribution parameters, the location-scale
transformation and the possibility to compute it from cdf's
when quantile functions are not available.


\section{Value at Risk and Expected Shortfall}

%To avoid confusion, we give the mathematical definitions here.
We use the traditional definition of VaR as the negated lower quantile.  More specifically,
let $Y$ be the variable of interes, such as return on a financial asset. Suppose that it is
modelled as a random variable with distribution function $F^{Y}(y)$. Then VaR is defined as%
\footnote{Beware that a definition without negation is also used in both the literature and
  in the R packages.}
\begin{equation*}
  \VaR{Y}  = - q_{\alpha}^{Y}
  .
\end{equation*}
where $q_{\alpha}^{Y}$ is the lower $\alpha$-quantile of $Y$, i.e. we have
\begin{equation*}
  \alpha = F(q_{\alpha}^{Y}) = P(Y \le q_{\alpha}^{Y})
  .
\end{equation*}
Typical values for $\alpha$ are $0.05$ and $0.01$.
Equivalently, we could write\footnote{This equation shows why some authors use the ``big numbers'', e.g. 0.95 and 0.99, in the notation for the VaR.}
\begin{equation*}
  \VaR{Y}  = q_{1-\alpha}^{-Y}
  .
\end{equation*}
The expected shortfall is the (partial) expectation of $\VaR{Y}$:
\begin{equation*}
  \ES{Y} = \frac{1}{\alpha}\int_{0}^{\alpha}\VaR[\gamma]{Y} d\gamma
  .
\end{equation*}

Suppose now that $Y$ is obtained from another random variable, $X$, by a linear transformation:
\begin{equation*}
  Y = a + bX
\end{equation*}
When this is the case, there is a simple relation between the VaR's and ES's of $Y$ and $X$:
\begin{align*}
  \VaR{Y} &= -a + b \VaR{X} \\
   \ES{Y} &= -a + b \ES{X}
\end{align*}
In practice, $X$ is often chosen to be standardised but this is not necessary. Note also that
if a bunch of VaR's and ES's are needed, say for normally distributed variables, this can be
computed very efficiently using this property.

\section{VaR}
\label{sec:var}

Here we compute the VaR associated with a standard normal r.v.
The three variants are equivalent since 0.05 and \code{"qf"} are default for the last two arguments.
<<>>=
cvar::VaR(qnorm, p_loss = 0.05, dist.type = "qf")
cvar::VaR(qnorm, p_loss = 0.05)
cvar::VaR(qnorm)
@

\code{x} can be a vector:
<<>>=
cvar::VaR(qnorm, p_loss = c(0.01, 0.05))
@

Let's set some more realistic values for the parameters of the normal distribution.  Suppose
that the daily returns on some stock have sample mean $0.006408553$ and sample variance
$0.0004018977$. Then $N(0.006408553, 0.0004018977)$ can be taken as normal distribution
fitted to the data.
<<>>=
muA <- 0.006408553
sigma2A <- 0.0004018977
@
This computes VaR using the fitted normal distribution:
<<>>=
res1 <- cvar::VaR(qnorm, p_loss = 0.05, mean = muA, sd = sqrt(sigma2A))
@

For the normal disribution we could also use the intercept-slope arguments, since the
parameters are precisely the intercept and the slope:
<<>>=
res2 <- cvar::VaR(qnorm, p_loss = 0.05, intercept = muA, slope = sqrt(sigma2A))
abs((res2 - res1)) # 0, intercept/slope equivalent to mean/sd
@

If we compute VaR using the cdf, the intercept-slope method has some numerical advantage:
<<>>=
## with cdf the precision depends on solving an equation
res1a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = muA, sd = sqrt(sigma2A))
res2a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = muA, slope = sqrt(sigma2A))
abs((res1a - res2)) # 3.287939e-09
abs((res2a - res2)) # 5.331195e-11, intercept/slope better numerically
@
Of course, it is almost always better to use the quantile function when it is available. 

We can use smaller tollerance to improve the precision
(the value \verb+.Machine$double.eps^0.75+ = \Sexpr{.Machine$double.eps^0.75} is probably excessive):
<<>>=
## as above, but increase the precision, this is probably excessive
res1b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = muA, sd = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
res2b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = muA, slope = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
abs((res1b - res2)) # 6.938894e-18 # both within machine precision
abs((res2b - res2)) # 1.040834e-16
@

The relative precision is also good.
<<>>=
abs((res1b - res2)/res2) # 2.6119e-16 # both within machine precision
abs((res2b - res2)/res2) # 3.91785e-15
@



For examples with vectorised distribution parameters we use data from
\pkg{PerformanceAnalytics}.

Compute means and variances of the variables in a data frame and store them in vectors:
<<>>=
## if(require("PerformanceAnalytics")){
data(edhec, package = "PerformanceAnalytics")
mu <- apply(edhec, 2, mean)
sigma2 <- apply(edhec, 2, var)
musigma2 <- cbind(mu, sigma2)
@

We compute VaR using \code{PerformanceAnalytics::VaR}:
<<>>=
## analogous calc. with PerformanceAnalytics::VaR
vPA <- apply(musigma2, 1, function(x)
    PerformanceAnalytics::VaR(p = .95, method = "gaussian", invert = FALSE,
                              mu = x[1], sigma = x[2], weights = 1))
@
The results below compare to the value, \code{vPA}, obtained here.

The computations below are similar to the previous example but the distribution parameters
are vectors. The first pair of results are numerically the same:
<<>>=
vAz1 <- cvar::VaR(qnorm, p_loss = 0.05, mean = mu, sd = sqrt(sigma2))
vAz2 <- cvar::VaR(qnorm, p_loss = 0.05, intercept = mu, slope = sqrt(sigma2))
max(abs((vPA - vAz1))) # 5.551115e-17
max(abs((vPA - vAz2))) #   ""
@

Computing VaR from cdf shows some advantage for the location-scale method:
<<>>=
vAz1a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = mu, sd = sqrt(sigma2))
vAz2a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = mu, slope = sqrt(sigma2))
max(abs((vPA - vAz1a))) # 3.287941e-09
max(abs((vPA - vAz2a))) #  1.465251e-10, intercept/slope better
@

The advantage remains for smaller tolerance:
<<>>=
vAz1b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = mu, sd = sqrt(sigma2),
                   tol = .Machine$double.eps^0.75)
vAz2b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = mu, slope = sqrt(sigma2),
                   tol = .Machine$double.eps^0.75)
max(abs((vPA - vAz1b))) # 4.374869e-13
max(abs((vPA - vAz2b))) # 3.330669e-16
@


\section{Expected shortfall}
\label{sec:expected-shortfall}

\code{ES} has essentially the same arguments as \code{VaR}. The examples below are obtained
almost literally by replacing the calls to \code{VaR} with \code{ES()} ones. 

Here we compute the VaR associated with a standard normal r.v.
The three variants are equivalent since 0.5 and \code{"qf"} are default for the last two arguments.
<<>>=
cvar::ES(qnorm, p_loss = 0.05, dist.type = "qf")
cvar::ES(qnorm, p_loss = 0.05)
cvar::ES(qnorm)
@

\code{x} can be a vector:
<<>>=
cvar::ES(qnorm, p_loss = c(0.01, 0.05))
@

Let's set some more realistic values for the parameters of the normal distribution:
<<>>=
muA <- 0.006408553
sigma2A <- 0.0004018977
@

This demonstrates the use of \code{intercept} and \code{slope}. These arguments use the
linear transformation property discussed in the theoretical section. For the normal
disribution the parameters are precisely the intercept and the slope:
<<>>=
res1 <- cvar::ES(qnorm, p_loss = 0.05, mean = muA, sd = sqrt(sigma2A))
res2 <- cvar::ES(qnorm, p_loss = 0.05, intercept = muA, slope = sqrt(sigma2A))
abs((res2 - res1)) 
@

If we compute ES using the cdf, the intercept slope method has some numerical advantage:
<<>>=
## with cdf the precision depends on solving an equation
res1a <- cvar::ES(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = muA, sd = sqrt(sigma2A))
res2a <- cvar::ES(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = muA, slope = sqrt(sigma2A))
abs((res1a - res2)) # 
abs((res2a - res2)) # intercept/slope better numerically
@

We can use smaller tollerance to improve the precision
(the value \verb+.Machine$double.eps^0.75+ = \Sexpr{.Machine$double.eps^0.75} is probably excessive):
<<>>=
## as above, but increase the precision, this is probably excessive
res1b <- cvar::ES(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = muA, sd = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
res2b <- cvar::ES(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = muA, slope = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
abs((res1b - res2))
abs((res2b - res2))
@

The relative precision is also good.
<<>>=
abs((res1b - res2)/res2)
abs((res2b - res2)/res2)
@

\newpage{}
\bibliography{REFERENCES}

\end{document}

% preamble taken from the vignette in package strucchange
%
\documentclass[nojss,article]{jss}
\usepackage[T1]{fontenc}
%\usepackage{a4wide}
\usepackage[left=2cm,right=2cm,bottom=15mm]{geometry}
\usepackage{graphicx,color,alltt}
\usepackage[authoryear,round,longnamesfirst]{natbib}
\usepackage{hyperref}

\usepackage{amsmath, amsfonts}
% \usepackage{Sweave}
%\definecolor{Red}{rgb}{0.7,0,0}
%\definecolor{Blue}{rgb}{0,0,0.8}
%\definecolor{hellgrau}{rgb}{0.55,0.55,0.55}

%\newcommand{\E}{\mbox{$\mathsf{E}$}}
%\newcommand{\VAR}{\mbox{$\mathsf{VAR}$}}
%\newcommand{\COV}{\mbox{$\mathsf{COV}$}}
%\newcommand{\p}{\mbox{$\mathsf{P}$}}
%\newcommand{\email}[1]{\href{mailto:#1}{\normalfont\texttt{#1}}}
%\newenvironment{smallexample}{\begin{alltt}\small}{\end{alltt}}

%\setlength{\parskip}{0.5ex plus0.1ex minus0.1ex}
%\setlength{\parindent}{0em}
%
%\bibpunct{(}{)}{;}{a}{}{,}
%
%\newcommand{\ui}{\underline{i}}
%\newcommand{\oi}{\overline{\imath}}

%\RequirePackage{color}
%\definecolor{Red}{rgb}{0.5,0,0}
%\definecolor{Blue}{rgb}{0,0,0.5}
%\definecolor{hellgrau}{rgb}{0.55,0.55,0.55}
%
%\hypersetup{%
%hyperindex,%
%colorlinks,%
%linktocpage,%
%plainpages=false,%
%linkcolor=Blue,%
%citecolor=Blue,%
%urlcolor=Red,%
%pdfstartview=Fit,%
%pdfview={XYZ null null null}%
%}


% SweaveOpts{engine=R,eps=FALSE}
%\VignetteIndexEntry{Brief guide to R package cvar}
%\VignetteDepends{cvar}
%\VignetteKeywords{expected shortfall, ES, CVaR, VaR, value at risk}
%\VignettePackage{cvar}


<<echo=FALSE>>=
library(cvar)
pd <- packageDescription("cvar")
@

\newcommand{\VaR}[2][\alpha]{\text{VaR}_{#1}(#2)}
\newcommand{\ES}[2][\alpha]{\text{ES}_{#1}(#2)}


\title{Brief guide to R package \pkg{cvar}}
\author{Georgi N. Boshnakov \\ University of Manchester }
\Plainauthor{Georgi N. Boshnakov}

\Address{
  Georgi N. Boshnakov\\
  School of Mathematics\\
  The University of Manchester\\
  Oxford Road, Manchester M13 9PL, UK\\
  URL: \url{http://www.maths.manchester.ac.uk/~gb/}
}

%\date{}
%\maketitle


\Abstract{
Package \pkg{cvar} is a small package with, essentially, two
functions --- \code{ES()} for computing the expected shortfall
and \code{VaR()} for Value at Risk.  The user specifies the
distribution by supplying one of the functions that define a
continuous distribution---currently this can be a quantile
function (qf), cumulative distribution function (cdf) or
probability density function (pdf). Virtually any continuous
distribution can be specified.

This vignette is part of package \pkg{cvar}, version~\Sexpr{pd$Version}. %$
}

\Keywords{expected shortfall, ES, CVaR, VaR, value at risk}
\Plainkeywords{expected shortfall, ES, CVaR, VaR, value at risk}


\begin{document}

\section{Introduction}
\label{sec:introduction}

There is a huge number of functions for computations with
distributions in core R and in contributed packages. Pdf's,
cdf's, quantile functions and random number generators are
covered comprehensively. The coverage of expected shortfall is
more patchy but a large collection of distributions, including
functions for expected shortfall, is provided by
\citet{VaRES2013}.
\citet{PerformanceAnalytics2018} and
\citet{actuarJSS2008} provide packages
covering comprehensively various aspects of risk measurement,
including some functions for expected shortfall.

Package \pkg{cvar} is a small package with, essentially two
functions --- \code{ES} for computing the expected shortfall
and \code{VaR} for Value at Risk.  The user specifies the
distribution by supplying one of the functions that define a
continuous distribution---currently this can be a quantile
function (qf), cumulative distribution function (cdf) or
probability density function (pdf). Virtually any continuous
distribution can be specified.
The computations are done directly from the definitions, see e.g. 
\citet{acerbi2002expected}.

The functions are vectorised over the parameters of the
distributions, making bulk computations more convenient, for
example for forecasting or model evaluation.

The name of this package, "cvar", comes from \emph{Conditional
Value at Risk} (CVaR), which is an alternative term for
expected shortfall.

We chose to use the standard names \code{ES} and \code{VaR},
despite the possibility for name clashes with same named
functions in other packages, rather than invent possibly
difficult to remember alternatives. Just call the functions as
\code{cvar::ES} and \code{cvar::VaR} if necessary.

Locations-scale transformations can be specified separately
from the other distribution parameters. This is useful when
such parameters are not provided directly by the distribution
at hand. The use of these parameters often leads to more
efficient computations and better numerical accuracy even if
the distribution has its own parameters for this purpose. Some
of the examples for \code{VaR} and \code{ES} illustrate this
for the Gaussian distribution.

Since VaR is a quantile, functions computing it for a given
distribution are convenience functions. \code{VaR} exported by
\pkg{cvar} could be attractive in certain workflows because of
its vectorised distribution parameters, the location-scale
transformation and the possibility to compute it from cdf's
when quantile functions are not available.


\section{Value at Risk and Expected Shortfall}

%To avoid confusion, we give the mathematical definitions here.
We use the traditional definition of VaR as the negated lower quantile.  More specifically,
let $Y$ be the variable of interes, such as return on a financial asset. Suppose that it is
modelled as a random variable with distribution function $F^{Y}(y)$. Then VaR is defined as%
\footnote{Beware that a definition without negation is also used in both the literature and
  in the R packages.}
\begin{equation*}
  \VaR{Y}  = - q_{\alpha}^{Y}
  .
\end{equation*}
where $q_{\alpha}^{Y}$ is the lower $\alpha$-quantile of $Y$, i.e. we have
\begin{equation*}
  \alpha = F(q_{\alpha}^{Y}) = P(Y \le q_{\alpha}^{Y})
  .
\end{equation*}
Typical values for $\alpha$ are $0.05$ and $0.01$.
Equivalently, we could write\footnote{This equation shows why some authors use the ``big numbers'', e.g. 0.95 and 0.99, in the notation for the VaR.}
\begin{equation*}
  \VaR{Y}  = q_{1-\alpha}^{-Y}
  .
\end{equation*}
The expected shortfall is the (partial) expectation of $\VaR{Y}$:
\begin{equation*}
  \ES{Y} = \frac{1}{\alpha}\int_{0}^{\alpha}\VaR[\gamma]{Y} d\gamma
  .
\end{equation*}

Suppose now that $Y$ is obtained from another random variable, $X$, by a linear transformation:
\begin{equation*}
  Y = a + bX
\end{equation*}
When this is the case, there is a simple relation between the VaR's and ES's of $Y$ and $X$:
\begin{align*}
  \VaR{Y} &= -a + b \VaR{X} \\
   \ES{Y} &= -a + b \ES{X}
\end{align*}
In practice, $X$ is often chosen to be standardised but this is not necessary. Note also that
if a bunch of VaR's and ES's are needed, say for normally distributed variables, this can be
computed very efficiently using this property.

\section{VaR}
\label{sec:var}

Here we compute the VaR associated with a standard normal r.v.
The three variants are equivalent since 0.05 and \code{"qf"} are default for the last two arguments.
<<>>=
cvar::VaR(qnorm, p_loss = 0.05, dist.type = "qf")
cvar::VaR(qnorm, p_loss = 0.05)
cvar::VaR(qnorm)
@

\code{x} can be a vector:
<<>>=
cvar::VaR(qnorm, p_loss = c(0.01, 0.05))
@

Let's set some more realistic values for the parameters of the normal distribution.  Suppose
that the daily returns on some stock have sample mean $0.006408553$ and sample variance
$0.0004018977$. Then $N(0.006408553, 0.0004018977)$ can be taken as normal distribution
fitted to the data.
<<>>=
muA <- 0.006408553
sigma2A <- 0.0004018977
@
This computes VaR using the fitted normal distribution:
<<>>=
res1 <- cvar::VaR(qnorm, p_loss = 0.05, mean = muA, sd = sqrt(sigma2A))
@

For the normal disribution we could also use the intercept-slope arguments, since the
parameters are precisely the intercept and the slope:
<<>>=
res2 <- cvar::VaR(qnorm, p_loss = 0.05, intercept = muA, slope = sqrt(sigma2A))
abs((res2 - res1)) # 0, intercept/slope equivalent to mean/sd
@

If we compute VaR using the cdf, the intercept-slope method has some numerical advantage:
<<>>=
## with cdf the precision depends on solving an equation
res1a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = muA, sd = sqrt(sigma2A))
res2a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = muA, slope = sqrt(sigma2A))
abs((res1a - res2)) # 3.287939e-09
abs((res2a - res2)) # 5.331195e-11, intercept/slope better numerically
@
Of course, it is almost always better to use the quantile function when it is available. 

We can use smaller tollerance to improve the precision
(the value \verb+.Machine$double.eps^0.75+ = \Sexpr{.Machine$double.eps^0.75} is probably excessive):
<<>>=
## as above, but increase the precision, this is probably excessive
res1b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = muA, sd = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
res2b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = muA, slope = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
abs((res1b - res2)) # 6.938894e-18 # both within machine precision
abs((res2b - res2)) # 1.040834e-16
@

The relative precision is also good.
<<>>=
abs((res1b - res2)/res2) # 2.6119e-16 # both within machine precision
abs((res2b - res2)/res2) # 3.91785e-15
@



For examples with vectorised distribution parameters we use data from
\pkg{PerformanceAnalytics}.

Compute means and variances of the variables in a data frame and store them in vectors:
<<>>=
## if(require("PerformanceAnalytics")){
data(edhec, package = "PerformanceAnalytics")
mu <- apply(edhec, 2, mean)
sigma2 <- apply(edhec, 2, var)
musigma2 <- cbind(mu, sigma2)
@

We compute VaR using \code{PerformanceAnalytics::VaR}:
<<>>=
## analogous calc. with PerformanceAnalytics::VaR
vPA <- apply(musigma2, 1, function(x)
    PerformanceAnalytics::VaR(p = .95, method = "gaussian", invert = FALSE,
                              mu = x[1], sigma = x[2], weights = 1))
@
The results below compare to the value, \code{vPA}, obtained here.

The computations below are similar to the previous example but the distribution parameters
are vectors. The first pair of results are numerically the same:
<<>>=
vAz1 <- cvar::VaR(qnorm, p_loss = 0.05, mean = mu, sd = sqrt(sigma2))
vAz2 <- cvar::VaR(qnorm, p_loss = 0.05, intercept = mu, slope = sqrt(sigma2))
max(abs((vPA - vAz1))) # 5.551115e-17
max(abs((vPA - vAz2))) #   ""
@

Computing VaR from cdf shows some advantage for the location-scale method:
<<>>=
vAz1a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = mu, sd = sqrt(sigma2))
vAz2a <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = mu, slope = sqrt(sigma2))
max(abs((vPA - vAz1a))) # 3.287941e-09
max(abs((vPA - vAz2a))) #  1.465251e-10, intercept/slope better
@

The advantage remains for smaller tolerance:
<<>>=
vAz1b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = mu, sd = sqrt(sigma2),
                   tol = .Machine$double.eps^0.75)
vAz2b <- cvar::VaR(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = mu, slope = sqrt(sigma2),
                   tol = .Machine$double.eps^0.75)
max(abs((vPA - vAz1b))) # 4.374869e-13
max(abs((vPA - vAz2b))) # 3.330669e-16
@


\section{Expected shortfall}
\label{sec:expected-shortfall}

\code{ES} has essentially the same arguments as \code{VaR}. The examples below are obtained
almost literally by replacing the calls to \code{VaR} with \code{ES()} ones. 

Here we compute the VaR associated with a standard normal r.v.
The three variants are equivalent since 0.5 and \code{"qf"} are default for the last two arguments.
<<>>=
cvar::ES(qnorm, p_loss = 0.05, dist.type = "qf")
cvar::ES(qnorm, p_loss = 0.05)
cvar::ES(qnorm)
@

\code{x} can be a vector:
<<>>=
cvar::ES(qnorm, p_loss = c(0.01, 0.05))
@

Let's set some more realistic values for the parameters of the normal distribution:
<<>>=
muA <- 0.006408553
sigma2A <- 0.0004018977
@

This demonstrates the use of \code{intercept} and \code{slope}. These arguments use the
linear transformation property discussed in the theoretical section. For the normal
disribution the parameters are precisely the intercept and the slope:
<<>>=
res1 <- cvar::ES(qnorm, p_loss = 0.05, mean = muA, sd = sqrt(sigma2A))
res2 <- cvar::ES(qnorm, p_loss = 0.05, intercept = muA, slope = sqrt(sigma2A))
abs((res2 - res1)) 
@

If we compute ES using the cdf, the intercept slope method has some numerical advantage:
<<>>=
## with cdf the precision depends on solving an equation
res1a <- cvar::ES(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = muA, sd = sqrt(sigma2A))
res2a <- cvar::ES(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = muA, slope = sqrt(sigma2A))
abs((res1a - res2)) # 
abs((res2a - res2)) # intercept/slope better numerically
@

We can use smaller tollerance to improve the precision
(the value \verb+.Machine$double.eps^0.75+ = \Sexpr{.Machine$double.eps^0.75} is probably excessive):
<<>>=
## as above, but increase the precision, this is probably excessive
res1b <- cvar::ES(pnorm, p_loss = 0.05, dist.type = "cdf",
                   mean = muA, sd = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
res2b <- cvar::ES(pnorm, p_loss = 0.05, dist.type = "cdf",
                   intercept = muA, slope = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
abs((res1b - res2))
abs((res2b - res2))
@

The relative precision is also good.
<<>>=
abs((res1b - res2)/res2)
abs((res2b - res2)/res2)
@

\newpage{}
\bibliography{REFERENCES}

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cvar-package.R
\docType{package}
\name{cvar-package}
\alias{cvar-package}
\alias{cvar}
\title{Compute Conditional Value-at-Risk and Value-at-Risk}
\description{
Compute expected shortfall (ES) and Value at Risk (VaR)
    from a quantile function, distribution function, random number
    generator or probability density function.  ES is also known as
    Conditional Value at Risk (CVaR). Virtually any continuous
    distribution can be specified.  The functions are vectorised over
    the arguments. Some support for GARCH models is provided, as well.
}
\details{
There is a huge number of functions for computations with
    distributions in core \R and in contributed packages. Pdf's,
    cdf's, quantile functions and random number generators are
    covered comprehensively. The coverage of expected shortfall is
    more patchy but a large collection of distributions, including
    functions for expected shortfall, is provided by
    \insertCite{VaRES2013;textual}{cvar}.
    \insertCite{PerformanceAnalytics2018;textual}{cvar} and
    \insertCite{actuarJSS2008;textual}{cvar} provide packages
    covering comprehensively various aspects of risk measurement,
    including some functions for expected shortfall.

    Package \pkg{cvar} is a small package with, essentially, two main
    functions --- \code{ES} for computing the expected shortfall
    and \code{VaR} for Value at Risk.  The user specifies the
    distribution by supplying one of the functions that define a
    continuous distribution---currently this can be a quantile
    function (qf), cumulative distribution function (cdf) or
    probability density function (pdf). Virtually any continuous
    distribution can be specified.

    The functions are vectorised over the parameters of the
    distributions, making bulk computations more convenient, for
    example for forecasting or model evaluation.

    The name of this package, "cvar", comes from \emph{Conditional
    Value at Risk} (CVaR), which is an alternative term for
    expected shortfall.

    We chose to use the standard names \code{ES} and \code{VaR},
    despite the possibility for name clashes with same named
    functions in other packages, rather than invent possibly
    difficult to remember alternatives. Just call the functions as
    \code{cvar::ES} and \code{cvar::VaR} if necessary.

    Locations-scale transformations can be specified separately
    from the other distribution parameters. This is useful when
    such parameters are not provided directly by the distribution
    at hand. The use of these parameters often leads to more
    efficient computations and better numerical accuracy even if
    the distribution has its own parameters for this purpose. Some
    of the examples for \code{VaR} and \code{ES} illustrate this
    for the Gaussian distribution.

    Since VaR is a quantile, functions computing it for a given
    distribution are convenience functions. \code{VaR} exported by
    \pkg{cvar} could be attractive in certain workflows because of
    its vectorised distribution parameters, the location-scale
    transformation, and the possibility to compute it from cdf's
    when quantile functions are not available.

    Some support for GARCH models is provided, as well. It is
    currently under development, see \code{\link{predict.garch1c1}}
    for current functionality.

    In practice, we may need to compute VaR associated with data. The distribution comes
    from fitting a model. In the simplest case, we fit a distribution to the data,
    assuming that the sample is i.i.d. For example, a normal distribution \eqn{N(\mu,
    \sigma^2)} can be fitted using the sample mean and sample variance as estimates of the
    unknown parameters \eqn{\mu} and \eqn{\sigma^2}, see section \sQuote{Examples}. For other
    common distributions there are specialised functions to fit their parameters and if
    not, general optimisation routines can be used. More soffisticated models may be used,
    even time series models such as GARCH and mixture autoregressive models.
}
\examples{
## see the examples for ES(), VaR(), predict.garch1c1()

}
\references{
\insertAllCited{}
}
\seealso{
\code{\link{ES}},
  \code{\link{VaR}}
}
\author{
Georgi N. Boshnakov
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/VaR.R
\name{ES}
\alias{ES}
\alias{ES.default}
\alias{ES.numeric}
\title{Compute expected shortfall (ES) of distributions}
\usage{
ES(dist, p_loss = 0.05, ...)

\method{ES}{default}(
  dist,
  p_loss = 0.05,
  dist.type = "qf",
  qf,
  ...,
  intercept = 0,
  slope = 1,
  control = list(),
  x
)

\method{ES}{numeric}(
  dist,
  p_loss = 0.05,
  dist.type = "qf",
  qf,
  ...,
  intercept = 0,
  slope = 1,
  control = list(),
  x
)
}
\arguments{
\item{dist}{specifies the distribution whose ES is computed, usually a function or a name
of a function computing quantiles, cdf, pdf, or a random number generator, see
Details.}

\item{p_loss}{level, default is 0.05.}

\item{...}{passed on to \code{dist}.}

\item{dist.type}{a character string specifying what is computed by \code{dist}, such as
"qf" or "cdf".}

\item{qf}{quantile function, only used if \code{dist.type = "pdf"}.}

\item{intercept, slope}{compute ES for the linear transformation \code{intercept +
slope*X}, where \code{X} has distribution specified by \code{dist}, see Details.}

\item{control}{additional control parameters for the numerical integration routine.}

\item{x}{deprecated and will soon be removed. \code{x} was renamed to \code{p_loss},
please use the latter.}
}
\value{
a numeric vector
}
\description{
Compute the expected shortfall for a distribution.
}
\details{
\code{ES} computes the expected shortfall for distributions specified by the
    arguments. \code{dist} is typically a function (or the name of one). What \code{dist}
    computes is determined by \code{dist.type}, whose default setting is \code{"qf"} (the
    quantile function). Other possible settings of \code{dist.type} include \code{"cdf"}
    and \code{"pdf"}.  Additional arguments for \code{dist} can be given with the
    \code{"..."} arguments.

    Argument \code{dist} can also be a numeric vector. In that case the ES is computed,
    effectively, for the empirical cumulative distribution function (ecdf) of the
    vector. The ecdf is not created explicitly and the \code{\link[stats]{quantile}}
    function is used instead for the computation of VaR. Arguments in \code{"..."} are
    passed eventually to \code{quantile()} and can be used, for example, to select a
    non-defult method for the computation of quantiles.

    Except for the exceptions discussed below, a function computing VaR for the specified
    distribution is constructed and the expected shortfall is computed by numerically
    integrating it. The numerical integration can be fine-tuned with argument
    \code{control}, which should be a named list, see \code{\link{integrate}} for the
    available options.

    If \code{dist.type} is \code{"pdf"}, VaR is not computed, Instead, the partial
    expectation of the lower tail is computed by numerical integration of \code{x *
    pdf(x)}.  Currently the quantile function is required anyway, via argument \code{qf},
    to compute the upper limit of the integral. So, this case is mainly for testing and
    comparison purposes.


    A bunch of expected shortfalls is computed if argument \code{x} or any of the
    arguments in \code{"..."} are of length greater than one. They are recycled to equal
    length, if necessary, using the normal \R recycling rules.

    \code{intercept} and \code{slope} can be used to compute the expected shortfall for
    the location-scale transformation \code{Y = intercept + slope * X}, where the
    distribution of \code{X} is as specified by the other parameters and \code{Y} is the
    variable of interest. The expected shortfall of \code{X} is calculated and then
    transformed to that of \code{Y}. Note that the distribution of \code{X} doesn't need
    to be standardised, although it typically will.

    The \code{intercept} and the \code{slope} can be vectors. Using them may be
    particularly useful for cheap calculations in, for example, forecasting, where the
    predictive distributions are often from the same family, but with different location
    and scale parameters. Conceptually, the described treatment of \code{intercept} and
    \code{slope} is equivalent to recycling them along with the other arguments, but more
    efficiently.

    The names, \code{intercept} and \code{slope}, for the location and scale parameters
    were chosen for their expressiveness and to minimise the possibility for a clash with
    parameters of \code{dist} (e.g., the Gamma distribution has parameter \code{scale}).
}
\examples{
ES(qnorm)

## Gaussian
ES(qnorm, dist.type = "qf")
ES(pnorm, dist.type = "cdf")

## t-dist
ES(qt, dist.type = "qf", df = 4)
ES(pt, dist.type = "cdf", df = 4)

ES(pnorm, 0.95, dist.type = "cdf")
ES(qnorm, 0.95, dist.type = "qf")
## - VaRES::esnormal(0.95, 0, 1)
## - PerformanceAnalytics::ETL(p=0.05, method = "gaussian", mu = 0,
##                             sigma = 1, weights = 1)             # same

cvar::ES(pnorm, dist.type = "cdf")
cvar::ES(qnorm, dist.type = "qf")
cvar::ES(pnorm, 0.05, dist.type = "cdf")
cvar::ES(qnorm, 0.05, dist.type = "qf")

## this uses "pdf"
cvar::ES(dnorm, 0.05, dist.type = "pdf", qf = qnorm)


## this gives warning (it does more than simply computing ES):
## PerformanceAnalytics::ETL(p=0.95, method = "gaussian", mu = 0, sigma = 1, weights = 1)

## run this if VaRRES is present
\dontrun{
x <- seq(0.01, 0.99, length = 100)
y <- sapply(x, function(p) cvar::ES(qnorm, p, dist.type = "qf"))
yS <- sapply(x, function(p) - VaRES::esnormal(p))
plot(x, y)
lines(x, yS, col = "blue")
}

}
\seealso{
\code{\link{VaR}} for VaR,

    \code{\link[=predict.garch1c1]{predict}} for examples with fitted models
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/VaR.R
\name{VaR}
\alias{VaR}
\alias{VaR_qf}
\alias{VaR_cdf}
\alias{VaR.default}
\alias{VaR.numeric}
\title{Compute Value-at-Risk (VaR)}
\usage{
VaR(dist, p_loss = 0.05, ...)

VaR_qf(
  dist,
  p_loss = 0.05,
  ...,
  intercept = 0,
  slope = 1,
  tol = .Machine$double.eps^0.5,
  x
)

VaR_cdf(
  dist,
  p_loss = 0.05,
  ...,
  intercept = 0,
  slope = 1,
  tol = .Machine$double.eps^0.5,
  x
)

\method{VaR}{default}(
  dist,
  p_loss = 0.05,
  dist.type = "qf",
  ...,
  intercept = 0,
  slope = 1,
  tol = .Machine$double.eps^0.5,
  x
)

\method{VaR}{numeric}(dist, p_loss = 0.05, ..., intercept = 0, slope = 1, x)
}
\arguments{
\item{dist}{specifies the distribution whose ES is computed, usually a function or a name
of a function computing quantiles, cdf, pdf, or a random number generator, see
Details.}

\item{p_loss}{level, default is 0.05.}

\item{...}{passed on to \code{dist}.}

\item{intercept, slope}{compute VaR for the linear transformation \code{intercept +
slope*X}, where \code{X} has distribution specified by \code{dist}, see Details.}

\item{tol}{tollerance}

\item{x}{deprecated and will soon be removed. \code{x} was renamed to \code{p_loss},
please use the latter.}

\item{dist.type}{a character string specifying what is computed by \code{dist}, such as
"qf" or "cdf".}
}
\description{
\code{VaR} computes the Value-at-Risk of the distribution specified by the
    arguments. The meaning of the parameters is the same as in \code{\link{ES}}, including
    the recycling rules.
}
\details{
\code{VaR} is S3 generic. The meaning of the parameters for its default method is the
    same as in \code{\link{ES}}, including the recycling rules.

    \code{VaR_qf} and \code{VaR_cdf} are streamlined, non-generic, variants for the common
    case when the \code{"..."} parameters are scalar. The parameters \code{x},
    \code{intercept}, and \code{slope} can be vectors, as for \code{VaR}.

    Argument \code{dist} can also be a numeric vector. In that case the ES is computed,
    effectively, for the empirical cumulative distribution function (ecdf) of the
    vector. The ecdf is not created explicitly and the \code{\link[stats]{quantile}}
    function is used instead for the computation of VaR. Arguments in \code{"..."} are
    passed eventually to \code{quantile()} and can be used, for example, to select a
    non-defult method for the computation of quantiles.

    In practice, we may need to compute VaR associated with data. The distribution comes
    from fitting a model. In the simplest case, we fit a distribution to the data,
    assuming that the sample is i.i.d. For example, a normal distribution \eqn{N(\mu,
    \sigma^2)} can be fitted using the sample mean and sample variance as estimates of the
    unknown parameters \eqn{\mu} and \eqn{\sigma^2}, see section \sQuote{Examples}. For
    other common distributions there are specialised functions to fit their parameters and
    if not, general optimisation routines can be used. More soffisticated models may be
    used, even time series models such as GARCH and mixture autoregressive models.
}
\note{
We use the traditional definition of VaR as the negated lower quantile. For example,
    if \eqn{X} are returns on an asset, VAR\eqn{{}_\alpha}{_a} = \eqn{-q_\alpha}{-q_a},
    where \eqn{q_\alpha}{-q_a} is the lower \eqn{\alpha}{a} quantile of \eqn{X}.
    Equivalently, VAR\eqn{{}_\alpha}{_a} is equal to the lower \eqn{1-\alpha}{1-a}
    quantile of \eqn{-X}.
}
\examples{
cvar::VaR(qnorm, c(0.01, 0.05), dist.type = "qf")

## the following examples use these values, obtained by fitting a normal distribution to
## some data:
muA <- 0.006408553
sigma2A <- 0.0004018977

## with quantile function, giving the parameters directly in the call:
res1 <- cvar::VaR(qnorm, 0.05, mean = muA, sd = sqrt(sigma2A))
res2 <- cvar::VaR(qnorm, 0.05, intercept = muA, slope = sqrt(sigma2A))
abs((res2 - res1)) # 0, intercept/slope equivalent to mean/sd

## with quantile function, which already knows the parameters:
my_qnorm <- function(p) qnorm(p, mean = muA, sd = sqrt(sigma2A))
res1_alt <- cvar::VaR(my_qnorm, 0.05)
abs((res1_alt - res1))

## with cdf the precision depends on solving an equation
res1a <- cvar::VaR(pnorm, 0.05, dist.type = "cdf", mean = muA, sd = sqrt(sigma2A))
res2a <- cvar::VaR(pnorm, 0.05, dist.type = "cdf", intercept = muA, slope = sqrt(sigma2A))
abs((res1a - res2)) # 3.287939e-09
abs((res2a - res2)) # 5.331195e-11, intercept/slope better numerically

## as above, but increase the precision, this is probably excessive
res1b <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
                   mean = muA, sd = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
res2b <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
                   intercept = muA, slope = sqrt(sigma2A), tol = .Machine$double.eps^0.75)
abs((res1b - res2)) # 6.938894e-18 # both within machine precision
abs((res2b - res2)) # 1.040834e-16

## relative precision is also good
abs((res1b - res2)/res2) # 2.6119e-16 # both within machine precision
abs((res2b - res2)/res2) # 3.91785e-15


## an extended example with vector args, if "PerformanceAnalytics" is present
if (requireNamespace("PerformanceAnalytics", quietly = TRUE)) withAutoprint({
    data(edhec, package = "PerformanceAnalytics")
    mu <- apply(edhec, 2, mean)
    sigma2 <- apply(edhec, 2, var)
    musigma2 <- cbind(mu, sigma2)

    ## compute in 2 ways with cvar::VaR
    vAz1 <- cvar::VaR(qnorm, 0.05, mean = mu, sd = sqrt(sigma2))
    vAz2 <- cvar::VaR(qnorm, 0.05, intercept = mu, slope = sqrt(sigma2))

    vAz1a <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
                       mean = mu, sd = sqrt(sigma2))
    vAz2a <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
                       intercept = mu, slope = sqrt(sigma2))

    vAz1b <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
                   mean = mu, sd = sqrt(sigma2),
                   tol = .Machine$double.eps^0.75)
    vAz2b <- cvar::VaR(pnorm, 0.05, dist.type = "cdf",
                   intercept = mu, slope = sqrt(sigma2),
                   tol = .Machine$double.eps^0.75)

    ## analogous calc. with PerformanceAnalytics::VaR
    vPA <- apply(musigma2, 1, function(x)
        PerformanceAnalytics::VaR(p = .95, method = "gaussian", invert = FALSE,
                                  mu = x[1], sigma = x[2], weights = 1))
    ## the results are numerically the same
    max(abs((vPA - vAz1))) # 5.551115e-17
    max(abs((vPA - vAz2))) #   ""

    max(abs((vPA - vAz1a))) # 3.287941e-09
    max(abs((vPA - vAz2a))) #  1.465251e-10, intercept/slope better

    max(abs((vPA - vAz1b))) # 4.374869e-13
    max(abs((vPA - vAz2b))) # 3.330669e-16
})

}
\seealso{
\code{\link{ES}} for ES,

    \code{\link[=predict.garch1c1]{predict}} for examples with fitted models
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/garch.R
\name{predict.garch1c1}
\alias{predict.garch1c1}
\title{Prediction for GARCH(1,1) time series}
\usage{
\method{predict}{garch1c1}(object, n.ahead = 1, Nsim = 1000, eps, sigmasq, seed = NULL, ...)
}
\arguments{
\item{object}{an object from class \code{"garch1c1"}.}

\item{n.ahead}{maximum horizon (lead time) for prediction.}

\item{Nsim}{number of Monte Carlo simulations for simulation based quantities.}

\item{eps}{the time series to predict, only the last value is used.}

\item{sigmasq}{the (squared) volatilities, only the last value is used.}

\item{seed}{an integer, seed for the random number generator.}

\item{...}{currently not used.}
}
\value{
an object from S3 class \code{"predict_garch1c1"} containing
    the following components:
\item{eps}{point predictions (conditional expectations) of the time series (equal
    to zero for pure GARCH).}
\item{h}{point predictions (conditional expectations)of the squared volatilities.}
\item{model}{the model.}
\item{call}{the call.}
\item{pi_plugin}{Prediction intervals for the time series, based on plug-in
    distributions, see Details.}
\item{pi_sim}{Simulation based prediction intervals for the time series, see Details.}
\item{dist_sim}{simulation samples from the predictive distributions of the time
    series and the volatilties. }
}
\description{
Predict GARCH(1,1) time series.
}
\details{
Plug-in prediction intervals and predictive distributions are obtained by
inserting the predicted volatility in the conditional densities. For
predictions more than one lag ahead these are not the real predictive
distributions but the prediction intervals are usually adequate.

For simulation prediction intervals we generate a (large) number of
continuations of the given time series. Prediction intervals can be based on
sample quantiles. The generated samples are stored in the returned object and
can be used for further exploration of the predictive
distributions. \code{dist_sim$eps} contains the simulated future values of
the time series and \code{dist_sim$h} the corresponding (squared)
volatilities.  Both are matrices whose \code{i}-th rows contain the predicted
quantities for horizon \code{i}.

The random seed at the start of the simulations is saved in the returned
object.  A speficific seed can be requested with argument \code{seed}. In
that case the simulations are done with the specified seed and the old state
of the random number generator is restored before the function returns.
This setup is similar to \code{\link{sim_garch1c1}}.
}
\note{
This function is under development and may be changed.
}
\examples{
op <- options(digits = 4)

## set up a model and simulate a time series
mo <- GarchModel(omega = 0.4, alpha = 0.3, beta = 0.5)
a1 <- sim_garch1c1(mo, n = 1000, n.start = 100, seed = 20220305)

## predictions for T+1,...,T+5 (T = time of last value)
## Nsim is small to reduce the load on CRAN, usually Nsim is larger.
a.pred <- predict(mo, n.ahead = 5, Nsim = 1000, eps = a1$eps,
  sigmasq = a1$h, seed = 1234)

## preditions for the time series
a.pred$eps

## PI's for eps - plug-in and simulated
a.pred$pi_plugin
a.pred$pi_sim

## a DIY calculation of PI's using the simulated sample paths
t(apply(a.pred$dist_sim$eps, 1, function(x) quantile(x, c(0.025, 0.975))))

## further investigate the predictive distributions
t(apply(a.pred$dist_sim$eps, 1, function(x) summary(x)))

## compare predictive densities for horizons 2 and 5:
h2 <- a.pred$dist_sim$eps[2, ]
h5 <- a.pred$dist_sim$eps[5, ]
main <- "Predictive densities: horizons 2 (blue) and 5 (black)"
plot(density(h5), main = main)
lines(density(h2), col = "blue")

## predictions of sigma_t^2
a.pred$h

## plug-in predictions of sigma_t
sqrt(a.pred$h)

## simulation predictive densities (PD's) of sigma_t for horizons 2 and 5:
h2 <- sqrt(a.pred$dist_sim$h[2, ])
h5 <- sqrt(a.pred$dist_sim$h[5, ])
main <- "PD's of sigma_t for horizons 2 (blue) and 5 (black)"
plot(density(h2), col = "blue", main = main)
lines(density(h5))

## VaR and ES for different horizons
cbind(h = 1:5,
      VaR = apply(a.pred$dist_sim$eps, 1, function(x) VaR(x, c(0.05))),
      ES = apply(a.pred$dist_sim$eps, 1, function(x) ES(x, c(0.05))) )

## fit a GARCH(1,1) model to exchange rate data and predict
gmo1 <- fGarch::garchFit(formula = ~garch(1, 1), data = fGarch::dem2gbp,
  include.mean = FALSE, cond.dist = "norm", trace = FALSE)
mocoef <- gmo1@fit$par
mofitted <- GarchModel(omega = mocoef["omega"], alpha = mocoef["alpha1"],
  beta = mocoef["beta1"])
gmo1.pred <- predict(mofitted, n.ahead = 5, Nsim = 1000, eps = gmo1@data,
  sigmasq = gmo1@h.t, seed = 1234)
gmo1.pred$pi_plugin
gmo1.pred$pi_sim

op <- options(op) # restore options(digits)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/garch.R
\name{GarchModel}
\alias{GarchModel}
\title{Specify a GARCH model}
\usage{
GarchModel(model = list(), ..., model.class = NULL)
}
\arguments{
\item{model}{a GARCH model or a list.}

\item{...}{named arguments specifying the GARCH model.}

\item{model.class}{a class for the result. By default \code{GarchModel()}
decides the class of the result.}
}
\value{
an object from suitable GARCH-type class
}
\description{
Specify a GARCH model.
}
\details{
Argument \code{model} can be the result of a previous call to \code{GarchModel}.
Arguments in \code{"..."} overwrite current components of \code{model}.

\code{GarchModel} guarantees that code using it will continue to work
transparently for the user even if the internal represedtation of GARCH
models in this package is changed or additional functionality is added.
}
\examples{
## GARCH(1,1) with Gaussian innovations
mo1a <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5)
mo1b <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5, cond.dist = "norm")

## equivalently, the parameters can be given as a list
p1 <- list(omega = 1, alpha = 0.3, beta = 0.5)
mo1a_alt <- GarchModel(p1)
mo1b_alt <- GarchModel(p1, cond.dist = "norm")
stopifnot(identical(mo1a, mo1a_alt), identical(mo1b, mo1b_alt))

## additional arguments modify values already in 'model'
mo_alt <- GarchModel(p1, beta = 0.4)

## set also initial values
mo2 <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5, esp0 = - 1.5, h0 = 4.96)

##  GARCH(1,1) with standardised-t_5
mot <- GarchModel(omega = 1, alpha = 0.3, beta = 0.5, cond.dist = list("std", nu = 5))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/garch.R
\name{sim_garch1c1}
\alias{sim_garch1c1}
\title{Simulate GARCH(1,1) time series}
\usage{
sim_garch1c1(model, n, n.start = 0, seed = NULL)
}
\arguments{
\item{model}{a GARCH(1,1) model, an object obtained from \code{GarchModel}.}

\item{n}{the length of the generated time series.}

\item{n.start}{number of warm-up values, which are then dropped.}

\item{seed}{an integer to use for setting the random number generator.}
}
\value{
a list with components:
\item{eps}{the time series,}
\item{h}{the (squared) volatilities,}
\item{eta}{the standardised innovations,}
\item{model}{the GARCH(1,1) model,}
\item{.sim}{a list containing the parameters of the simulation,}
\item{call}{the call.}
}
\description{
Simulate GARCH(1,1) time series.
}
\details{
The simulated time series is in component \code{eps} of the returned value.
For exploration of algorithms and eestimation procedures, the volatilities
and the standardised innovations are also returned.

The random seed at the start of the simulations is saved in the returned
object.  A speficific seed can be requested with argument \code{seed}. In
that case the simulations are done with the specified seed and the old state
of the random number generator is restored before the function returns.
}
\note{
This function is under development and may be changed.
}
